from flask_wtf import FlaskForm
from sqlalchemy import func
from wtforms import PasswordField
from wtforms.fields import EmailField
from wtforms.validators import DataRequired
from wtforms.validators import Email
from wtforms.validators import EqualTo
from wtforms.validators import InputRequired
from wtforms.validators import Length
from wtforms.validators import ValidationError

from .models import User


class LoginForm(FlaskForm):
    email = EmailField(
        "email",
        [DataRequired(), Email(message="Not a valid email address.")],
        render_kw={"class": "form-control", "autocomplete": "off"},
    )
    password = PasswordField(
        "Password",
        [DataRequired()],
        render_kw={"class": "form-control", "autocomplete": "off"},
    )


class RegistrationForm(FlaskForm):
    """Registration Form"""

    email = EmailField(
        "email_label",
        [DataRequired(), Email(message="Not a valid email address.")],
    )

    password = PasswordField(
        "New Password",
        validators=[
            InputRequired("Password is required"),
            Length(
                min=6,
                max=25,
                message="Password must be between 6 and 25 characters",
            ),
            EqualTo("confirm", message="Passwords must match"),
        ],
    )
    confirm = PasswordField(
        "Repeat Password",
    )

    def validate_email(self, field):
        """
        Inline validator for email. Checks to see if a user object with
        entered email already present in the database

        Args:
            field : The form field that contains email data.

        Raises:
            ValidationError: if the username entered in the field is already
            in the database
        """
        user = User.query.filter(
            func.lower(User.email) == func.lower(field.data)
        ).scalar()

        if user is not None:
            raise ValidationError(f"email '{field.data}' is already in use.")



"""
test all the different configurations types in
config.py
"""
import pytest


class TestAppConfigs:
    @pytest.mark.parametrize("app_type", ["development"])
    def test_dev_app_config(self, app):
        config = app.config

        assert "ENV" in config
        assert config["ENV"] == "development"
        assert "DEBUG" in config
        assert config["DEBUG"] is True
        assert config["LOGIN_DISABLED"] is not None
        assert "SECRET_KEY" in config
        assert config["SECRET_KEY"] is not None
        assert "SQLALCHEMY_DATABASE_URI" in config
        assert config["SQLALCHEMY_DATABASE_URI"] is not None
        assert config["MAIL_SERVER"] in ["localhost", "console"]
        assert "MAIL_PORT" in config
        assert "MAIL_USERNAME" in config
        assert "MAIL_PASSWORD" in config
        assert "MAIL_DEFAULT_SENDER" in config

    # @pytest.mark.parametrize("app_type", ["production"])
    # def test_prod_app_config(self, app):
    #     """
    #     Test the productions configs. Environment variable
    #     configs(private configs) are loaded from .test.prod.env just
    #     to make sure these are loaded correctly
    #     """
    #     config = app.config

    #     assert "ENV" in config
    #     assert config["ENV"] == "production"
    #     assert "DEBUG" in config
    #     assert config["DEBUG"] is False
    #     assert "SECRET_KEY" in config
    #     assert config["SECRET_KEY"] == "secret"
    #     assert "EMAIL_CONFIRMATION_DISABLED" in config
    #     assert config["MAIL_SERVER"] not in ["localhost", "console"]
    #     assert "MAIL_PORT" in config
    #     assert "MAIL_USERNAME" in config
    #     assert config["MAIL_USERNAME"] == "foo@gmail.com"
    #     assert "MAIL_PASSWORD" in config
    #     assert config["MAIL_PASSWORD"] == "pass"
    #     assert "MAIL_DEFAULT_SENDER" in config
    #     assert config["MAIL_DEFAULT_SENDER"] == "foo@gmail.com"
    #     assert "SQLALCHEMY_DATABASE_URI" in config
    # assert (
    #     config["SQLALCHEMY_DATABASE_URI"]
    #     == "mysql+pymysql://db_username:db_password@db_host/db_name"
    # )



from functools import wraps

from flask import redirect
from flask import url_for
from flask_login import current_user


def check_confirmed(func):
    @wraps(func)
    def decorated_function(*args, **kwargs):
        if current_user.is_email_confirmed:
            return func(*args, **kwargs)
        return redirect(url_for("auth.unconfirmed"))

    return decorated_function



from datetime import datetime

from flask import session
from flask import url_for
from init import db
from modules.box__default.i18n.models import LangRecord

from shopyo.api.models import PkModel


class Page(PkModel):
    __tablename__ = "pages"

    created_date = db.Column(db.DateTime, default=datetime.now())
    title = db.Column(db.String(100))
    slug = db.Column(db.String(100))

    def insert_lang(self, lang, content):
        record = LangRecord(strid=self.get_strid(), lang=lang, string=content)
        record.save(commit=False)

    def set_lang(self, lang, content, commit=False):
        record = LangRecord.query.filter(
            LangRecord.strid == self.get_strid(), LangRecord.lang == lang
        ).first()
        if not record:
            record = LangRecord(strid=self.get_strid(), lang=lang, string=content)
            record.save(commit=commit)
        else:
            record.string = content
            record.update(commit=commit)

    def get_strid(self):
        if self.id is None:
            raise Exception("Cannot save page, id none")
        return f"page_{self.id}"

    def get_content(self, lang=None):
        if not lang:
            lang = session.get("yo_current_lang", "en")
        Page.query.get(self.id)
        record = LangRecord.query.filter(
            LangRecord.strid == self.get_strid(), LangRecord.lang == lang
        ).first()
        if record is None:
            return None

        return record.string

    def get_url(self, lang=None):
        if self.slug:
            if lang:
                return url_for("page.view_page", slug=self.slug, lang=lang)
            else:
                return url_for("page.view_page", slug=self.slug)
        else:
            return None

    def get_dashboard_url(self, lang=None):
        if self.slug:
            if lang:
                return url_for("page.view_page_dashboard", slug=self.slug, lang=lang)
            else:
                return url_for("page.view_page_dashboard", slug=self.slug)
        else:
            return None

    def get_langs(self):
        records = LangRecord.query.filter(LangRecord.strid == self.get_strid()).all()

        page_langs = [p.lang for p in records]

        return page_langs



"""
File conftest.py for auth testing contains pytest fixtures that are only in
box__default/auth module. Refer to https://docs.pytest.org/en/stable/fixture.html
for more details on pytest
"""
import pytest


@pytest.fixture
def email_config(request, flask_app):
    """
    pytest fixture for temporally changing the email related configs
    To remove the config pass "remove" in @pytest.parameterize. For
    setting value to the config pass the actual value. See test_email.py
    for usage

    Args:
        request (pytest obj): a built in by pytest object used to read
            incoming fixture arguments
        flask_app (flask app): flask app fixture

    """
    config_name, config_val = request.param
    old = flask_app.config[config_name]

    if config_val == "remove":
        del flask_app.config[config_name]
    else:
        flask_app.config[config_name] = config_val
        print(f"\n{config_name}: {config_val}\n")

    yield
    flask_app.config[config_name] = old



import pytest

from shopyo.api import validators


class TestValidators:
    """Tests validator methods"""

    @pytest.mark.parametrize(
        "url,expected",
        [
            ("google", False),
            ("www.google.com", True),
            ("https://www.google.com", True),
            ("localhost:3000", True),
            ("192.168.0.250", True),
            ("2001:0db8:85a3:0000:0000:8a2e:0370:7334", True),
        ],
    )
    def test_is_valid_url(self, url, expected):
        result = validators.is_valid_url(url)
        assert result == expected

    @pytest.mark.parametrize(
        "string,expected",
        [
            ("efwfwefwefwef", True),
            ("wehfbweur76tr46348tr", True),
            ("uwfehbuweify2874gr34_____________", True),
            ("^uefew.", False),
            ("wefwfwe/wfwefewf/ewfwef", False),
            ("""%^$^$£%£""£%"£""", False),
            ("""%^$^$£%£""£%"£__""", False),
            ("""%^$^$£%£""£%"£wdwqdqwd__""", False),
            ("""%^$^$£%£""£%"£423423423""", False),
        ],
    )
    def test_is_alpha_num_underscore(self, string, expected):
        result = validators.is_alpha_num_underscore(string)
        assert result == expected

    @pytest.mark.parametrize(
        "string,expected",
        [(" ", True), ("", True), (" adqwd", False), ("wefwefwef", False)],
    )
    def test_is_empty_str(self, string, expected):
        result = validators.is_empty_str(string)
        assert result == expected

    @pytest.mark.parametrize(
        "string,expected",
        [
            ("wefwf-wfwefwef-wefwef", True),
            ("-wefwef-ewfwef-wefwef-wefwef", True),
            ("-wefefwef-", True),
            ("-w6ef78wef687ewf6-wefwef-", True),
            ("---", True),
            ("/efiewf/", False),
            ("(ewfewf)", False),
        ],
    )
    def test_is_valid_slug(self, string, expected):
        result = validators.is_valid_slug(string)
        assert bool(result) == expected



# Please add your functional tests to this file.
from shopyo.api.assets import get_static


def test_static_module(test_client):
    response = test_client.get(get_static("tests", "file.css"))
    assert response.status_code == 200
    assert b"module tests file.css" in response.data


def test_static_box(test_client):
    response = test_client.get(get_static("box__tests/test1", "file.css"))
    assert response.status_code == 200
    assert b"box__tests/tests file.css" in response.data



import datetime
import json

from modules.box__default.auth.models import User


def add_admin(email, password):
    user = User()
    user.email = email
    user.password = password
    user.is_admin = True
    user.is_email_confirmed = True
    user.email_confirm_date = datetime.datetime.now()
    user.save()


def upload(verbose=False):
    with open("config.json") as config:
        config = json.load(config)
        add_admin(config["admin_user"]["email"], config["admin_user"]["password"])

        if verbose:
            print("[x] Added Admin User")



"""
file: api/tests/conftest.py
All pytest fixtures local only to the api/tests are placed here
"""
import os
import shutil
import tempfile

import pytest


@pytest.fixture
def cleandir():
    old_cwd = os.getcwd()
    newpath = tempfile.mkdtemp()
    os.chdir(newpath)
    yield
    os.chdir(old_cwd)
    shutil.rmtree(newpath)


@pytest.fixture
def restore_cwd():
    old = os.getcwd()
    yield
    os.chdir(old)


@pytest.fixture
def fake_foo_proj(tmp_path):
    """creates a fake shopyo like directory structure as shown below

    foo/
        foo/
            modules/
                bar/
                    static/
                        bar.css
                baz/
                    static/
                        baz.css
                box__bizhelp/
                    demo/
                        demo.py
                box__default/
                    foo/
                        static/
                            foo.css
                    foozoo/
                        foozoo.py
                    zoo/
                        static/
                            zoo.css
            static/

    Parameters
    ----------
    tmp_path : pathlib.Path
        built in pytest fixture which will provide a temporary directory unique
        to the test invocation, created in the base temporary directory.
    """
    # create the tmp_path/foo/foo
    project_path = tmp_path / "foo" / "foo"
    project_path.mkdir(parents=True)
    # create the static and modules inside foo/foo
    static_path = project_path / "static"
    module_path = project_path / "modules"
    static_path.mkdir()
    module_path.mkdir()
    # create the dummy boxes and modules
    demo_path = module_path / "box__bizhelp/demo/demo.py"
    foo_path = module_path / "box__default/foo/static/foo.css"
    zoo_path = module_path / "box__default/zoo/static/zoo.css"
    foozoo_path = module_path / "box__default/foozoo/foozoo.py"
    bar_path = module_path / "bar/static/bar.css"
    baz_path = module_path / "baz/model/baz.py"
    demo_path.parent.mkdir(parents=True)
    foo_path.parent.mkdir(parents=True)
    zoo_path.parent.mkdir(parents=True)
    foozoo_path.parent.mkdir(parents=True)
    bar_path.parent.mkdir(parents=True)
    baz_path.parent.mkdir(parents=True)
    demo_path.write_text("demo")
    foo_path.write_text("foo")
    zoo_path.write_text("zoo")
    foozoo_path.write_text("foozoo")
    bar_path.write_text("bar")
    baz_path.write_text("baz")
    # save cwd and chage to test project directory
    old = os.getcwd()
    os.chdir(project_path)
    yield project_path
    # restore old cwd directory
    os.chdir(old)



class TestApiTemplates:
    """
    Test templates functionalities
    """

    def test_yo_render(self, test_client):
        response = test_client.get("/render_demo")

        assert response.status_code == 200
        assert b"Fruit mango" in response.data



from flask import render_template

from shopyo.api.module import ModuleHelp
from shopyo.api.templates import yo_render

# from flask import url_for
# from flask import redirect
# from flask import flash
# from flask import request
#
# from shopyo.api.html import notify_success
# from shopyo.api.forms import flash_errors
# from shopyo.api.enhance import get_active_theme_dir
# from shopyo.api.enhance import get_setting
# from modules.box__ecommerce.shop.helpers import get_cart_data

mhelp = ModuleHelp(__file__, __name__)
globals()[mhelp.blueprint_str] = mhelp.blueprint
module_blueprint = globals()[mhelp.blueprint_str]


@module_blueprint.route("/")
def index():
    # cant be defined above but must be manually set each time
    # active_theme_dir = os.path.join(
    #     dirpath, "..", "..", "themes", get_setting("ACTIVE_FRONT_THEME")
    # )
    # module_blueprint.template_folder = active_theme_dir

    # return str(module_blueprint.template_folder)

    # return render_template(get_setting("ACTIVE_FRONT_THEME") + "/index.html")

    return render_template("www/index.html", get_static=get_static)


from shopyo.api.assets import get_static


@module_blueprint.route("/render_demo")
def render_demo():
    context = {"fruit": "mango"}
    return yo_render("blogus/render_demo.html", context)



import json

from init import db
from modules.box__default.settings.models import Settings


def add_setting(name, value):
    if Settings.query.filter_by(setting=name).first():
        s = Settings.query.get(name)
        s.value = value
        db.session.commit()
    else:
        s = Settings(setting=name, value=value)
        db.session.add(s)
        db.session.commit()


def upload(verbose=False):
    with open("config.json") as config:
        config = json.load(config)
        for name, value in config["settings"].items():
            add_setting(name, value)

        if verbose:
            print("[x] Added Dummy Settings")



from flask import redirect
from flask import request
from flask import url_for
from flask_admin import AdminIndexView
from flask_admin import expose
from flask_admin.contrib import sqla as flask_admin_sqla
from flask_login import current_user


class DefaultModelView(flask_admin_sqla.ModelView):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def is_accessible(self):
        return current_user.is_authenticated and current_user.is_admin

    def inaccessible_callback(self, name, **kwargs):
        # redirect to login page if user doesn't have access
        return redirect(url_for("auth.login", next=request.url))


class MyAdminIndexView(AdminIndexView):
    def is_accessible(self):
        return current_user.is_authenticated and current_user.is_admin

    def inaccessible_callback(self, name, **kwargs):
        # redirect to login page if user doesn't have access
        return redirect(url_for("auth.login", next=request.url))

    @expose("/")
    def index(self):
        if not current_user.is_authenticated and current_user.is_admin:
            return redirect(url_for("auth.login"))
        return super().index()

    @expose("/dashboard")
    def indexs(self):
        if not current_user.is_authenticated and current_user.is_admin:
            return redirect(url_for("auth.login"))
        return super().index()



from flask import session
from modules.box__default.i18n.helpers import lang_keys

from shopyo.api.module import ModuleHelp

# from flask import render_template
# from flask import url_for
# from flask import redirect
# from flask import flash
# from flask import request

# from shopyo.api.html import notify_success
# from shopyo.api.forms import flash_errors

mhelp = ModuleHelp(__file__, __name__)
globals()[mhelp.blueprint_str] = mhelp.blueprint
module_blueprint = globals()[mhelp.blueprint_str]


@module_blueprint.route("/")
def index():
    return mhelp.info["display_string"]


@module_blueprint.route("/set-lang", methods=["GET"])
def set_lang():
    set_to_lang = request.args.get("lang", "en")
    next_url = request.args.get("next", "/")

    if set_to_lang in lang_keys():
        session["yo_current_lang"] = set_to_lang
        session["yo_default_lang"] = set_to_lang

        return redirect(get_safe_url(next_url))


# If "dashboard": "/dashboard" is set in info.json
#
# @module_blueprint.route("/dashboard", methods=["GET"])
# def dashboard():

#     context = mhelp.context()

#     context.update({

#         })
#     return mhelp.render('dashboard.html', **context)



import json
import os

from flask import current_app
from flask import flash
from flask import render_template
from flask_login import login_required
from modules.box__default.appadmin.admin import admin_required
from modules.box__default.auth.decorators import check_confirmed

from shopyo.api.html import notify_success
from shopyo.api.module import ModuleHelp

mhelp = ModuleHelp(__file__, __name__)
globals()[mhelp.blueprint_str] = mhelp.blueprint
module_blueprint = globals()[mhelp.blueprint_str]


all_info = {}


@dashboard_blueprint.route("/")
@login_required
@check_confirmed
@admin_required
def index():
    context = {}

    for folder in os.listdir(os.path.join(current_app.config["BASE_DIR"], "modules")):
        if folder.startswith("__"):
            continue
        elif folder.startswith("box__"):
            for sub_folder in os.listdir(
                os.path.join(current_app.config["BASE_DIR"], "modules", folder)
            ):
                if sub_folder in ["dashboard"]:
                    continue
                if sub_folder.startswith("__"):  # ignore __pycache__
                    continue
                elif sub_folder.endswith(".json"):  # box_info.json
                    continue
                with open(
                    os.path.join(
                        current_app.config["BASE_DIR"],
                        "modules",
                        folder,
                        sub_folder,
                        "info.json",
                    )
                ) as f:
                    module_info = json.load(f)
                    all_info[sub_folder] = module_info
        else:
            if folder not in ["dashboard"]:
                with open(
                    os.path.join(
                        current_app.config["BASE_DIR"],
                        "modules",
                        folder,
                        "info.json",
                    )
                ) as f:
                    module_info = json.load(f)
                    all_info[folder] = module_info

    context["all_info"] = all_info
    flash(notify_success("Notif test"))
    return render_template("dashboard/index.html", **context)



from flask import flash

from shopyo.api.html import notify_warning


def flash_errors(form):
    """
    Auto flash errors from WKHtml forms
    Reqwires base module or similar notification
    mechanism

    Parameters
    ----------
    form: WKHtml form

    Returns
    -------
    None
    """
    for field, errors in form.errors.items():
        for error in errors:
            error_msg = "Error in the {} field - {}".format(
                getattr(form, field).label.text,
                error,
            )
            flash(notify_warning(error_msg))



"""
This file email.py contains functions for sending
text and html rendered emails asynchronously
"""
from threading import Thread

from flask import current_app
from flask import render_template
from flask_mailman import EmailMultiAlternatives


def _send_email_helper(app, msg):
    """
    Helper function used for sending email message

    Args:
        app (Flask): The flask app object
        msg (flask-mailman email object): any email/messsage object
            defined for flask-mailman. Example EmailMessage
    """
    with app.app_context():
        if (
            "MAIL_USERNAME" not in current_app.config
            or "MAIL_PASSWORD" not in current_app.config
            or current_app.config["MAIL_USERNAME"] is None
            or current_app.config["MAIL_PASSWORD"] is None
        ):
            print(
                "\nShopyo Error: MAIL_USERNAME, and/or MAIL_PASSWORD not configured\n"
            )
            return

        msg.send()


def send_async_email(to, subject, template, from_email=None, **kwargs):
    """
    Sends email anachronously i.e the function is non blocking.
    Assume email template is valid i.e it can be rendered using
    flask' render_template function and both .html and .txt
    email template files exits

    Args:
        to (String): recipient of the email
        subject (String): subject of the email
        template (String): template file path to be used in email body
        from_email (String, optional): sender of the email. If not set
            MAIL_DEFAULT_SENDER is used from config.
    """

    if from_email is None:
        if (
            "MAIL_DEFAULT_SENDER" not in current_app.config
            or current_app.config["MAIL_DEFAULT_SENDER"] is None
        ):
            print("\nShopyo Error: MAIL_DEFAULT_SENDER not configured\n")
            return

        from_email = current_app.config["MAIL_DEFAULT_SENDER"]

    app = current_app._get_current_object()
    template_txt = render_template(f"{template}.txt", **kwargs)
    template_html = render_template(f"{template}.html", **kwargs)

    msg = EmailMultiAlternatives(
        subject=subject,
        body=template_txt,
        from_email=from_email,
        to=[to],
    )
    msg.attach_alternative(template_html, "text/html")

    thr = Thread(target=_send_email_helper, args=[app, msg])
    thr.start()
    return thr



from init import db

from shopyo.api.models import PkModel


class LangRecord(PkModel):
    __tablename__ = "i18n_records"
    strid = db.Column(db.String(1024))
    lang = db.Column(db.String(10))
    string = db.Column(db.Text())



import os
import re

from wtforms.validators import ValidationError

# https://wtforms.readthedocs.io/en/2.3.x/validators/


def get_module_path_if_exists(name):
    root_path = os.getcwd()

    for folder in os.listdir(os.path.join(root_path, "modules")):
        module_path = os.path.join(root_path, "modules", folder)
        sub_module_path = os.path.join(module_path, name)

        if folder == name:
            return module_path

        if os.path.exists(sub_module_path):
            return sub_module_path

    return None


def is_alpha_num_underscore(name):
    """returns whether the given name contains only alphanumeric or underscore

    Parameters
    ----------
    name : str
        to value to check for alphanumeric or underscore

    Returns
    -------
    bool
        returns ``True`` if ``name`` is alphanumeric, ``False`` otherwise
    """
    return bool(re.match(r"^[A-Za-z0-9_]+$", name))


def is_empty_str(string):
    return string.strip() == ""


def is_valid_slug(text):
    # from validators package
    slug_regex = re.compile(r"^[-a-zA-Z0-9_]+$")
    return slug_regex.match(text)


def is_valid_url(url):
    protocol = r"^((?:http|ftp)s?://)?"
    domain = r"(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+[A-Z]{2,6}\.?|localhost)"
    ipv4 = r"(?:25[0-5]|2[0-4]\d|[0-1]?\d?\d)(?:\.(?:25[0-5]|2[0-4]\d|[0-1]?\d?\d)){3}"
    ipv6 = r"([a-f0-9:]+:+)+[a-f0-9]+"
    port = r"(?::\d+)?(?:/?|[/?]\S+)$"
    url_regex = re.compile(
        protocol + domain + "|" + ipv4 + "|" + ipv6 + port, re.IGNORECASE
    )
    return url_regex.match(url) is not None


def verify_slug(form, field):
    if not is_valid_slug(field.data):
        raise ValidationError(
            "Slugs can only contain alphabets, numbers and hyphens (-). eg. good-day-1"
        )



import os
import subprocess
import sys
from shutil import copytree

import pytest

from shopyo import __version__

pytestmark = pytest.mark.cli_integration


# @pytest.mark.usefixtures("restore_cwd")
# def test_initialise_after_new(tmp_path):
#     """run shopyo new inside a tmp directory foo, create a venv, install the
#     shopyo.tar.gz dependencies, then run `shopyo initialise`.
#     """
#     # go one level up to the cwd so we are are the root where
#     # setup.py exits
#     os.chdir("../")
#     # create the dist folder with shoypo-<version>.tar.gz file
#     subprocess.check_call([sys.executable, "setup.py", "sdist"])
#     # store all path names to be used later
#     dist_path = os.path.join(os.getcwd(), "dist")
#     shopyo_dist_name = f"shopyo-{__version__}.tar.gz"
#     project_path = tmp_path / "foo"
#     print(project_path)
#     # copy the shopyo dist to the test project path
#     copytree(dist_path, os.path.join(project_path, "dist"))
#     # change cwd to that of test project
#     os.chdir(project_path)
#     # create a new virtual environment(venv)
#     subprocess.check_call([sys.executable, "-m", "venv", "env"])
#     # store path for python and shopyo executable of venv for the case when OS
#     #  is Unix
#     python_env = os.path.join(os.getcwd(), "env", "bin", "python")
#     shopyo_env = os.path.join(os.getcwd(), "env", "bin", "shopyo")
#     # if OS is Windows, update the python and shopyo executable
#     if sys.platform == "win32":
#         python_env = os.path.join(os.getcwd(), "env", "Scripts", "python")
#         shopyo_env = os.path.join(os.getcwd(), "env", "Scripts", "shopyo")
#     # update pip of venv
#     subprocess.check_call([python_env, "-m", "pip", "install", "--upgrade", "pip"])
#     # install the shopyo package from dist added earlier
#     subprocess.check_call(
#         [python_env, "-m", "pip", "install", os.path.join("dist", shopyo_dist_name)]
#     )
#     # run shopyo help command followed by new command
#     subprocess.check_call(["shopyo", "--help"])
#     subprocess.check_call([shopyo_env, "new"])
#     # change the cwd to the newly created shopyo project
#     os.chdir(os.path.join(project_path, "foo"))
#     # initialise the project
#     subprocess.check_call([shopyo_env, "initialise"])
#     assert os.path.exists("shopyo.db")
#     assert os.path.exists("migrations")



"""
This file (test_auth_functional.py) contains the functional tests for
the `auth` blueprint.

These tests use GETs and POSTs to different endpoints to check
for the proper behavior of the `auth` module
"""
import json
import os
import threading

import pytest
import sqlalchemy
from flask import request
from flask import url_for
from modules.box__default.auth.models import User

dirpath = os.path.dirname(os.path.abspath(__file__))
module_path = os.path.dirname(dirpath)

module_info = None

with open(os.path.join(module_path, "info.json")) as f:
    module_info = json.load(f)


# class TestAuthInvalidAccess:
#     """
#     Test all auth routes for correct user authentication
#     """

#     routes_get = [
#         "/confirm/<token>",
#         "/resend",
#         "/unconfirmed",
#     ]

#     @pytest.mark.parametrize("route", routes_get)
#     def test_redirect_if_not_logged_in_get(self, test_client, route, auth):
#         auth.logout()
#         response = test_client.get(
#             f"{module_info['url_prefix']}{route}", follow_redirects=True
#         )

#         assert response.status_code == 200
#         assert request.path == url_for("auth.login")


class TestAuthEndpoints:
    """
    Test all auth routes' functionalities
    """

    def test_user_registration_page_renders(self, test_client):
        response = test_client.get(f"{module_info['url_prefix']}/register")

        assert response.status_code == 200
        assert b"Email" in response.data
        assert b"Password" in response.data
        assert b"Confirm Password" in response.data
        assert b"Register" in response.data

    def test_user_not_registered_on_invalid_form_submit(self, test_client):
        User.create(email="test@gmail.com", password="pass")
        data = {
            "email": "test@gmail.com",
            "password": "password",
            "confirm": "password",
        }

        response = test_client.post(
            f"{module_info['url_prefix']}/register",
            data=data,
            follow_redirects=True,
        )

        assert response.status_code == 200
        assert request.path == url_for("auth.register")

    def test_user_registration_is_case_insensitive(self, test_client):
        User.create(email="foo@bar.com", password="pass")
        data = {
            "email": "Foo@Bar.com",
            "password": "password",
            "confirm": "password",
        }

        response = test_client.post(
            f"{module_info['url_prefix']}/register",
            data=data,
            follow_redirects=True,
        )

        assert response.status_code == 200
        assert request.path == url_for("auth.register")

    # @pytest.mark.parametrize(
    #     "email_config",
    #     [
    #         ("EMAIL_CONFIRMATION_DISABLED", True),
    #     ],
    #     indirect=True,
    # )
    # def test_user_confirmed_if_email_disabled(self, test_client, email_config):
    #     data = {
    #         "email": "test@gmail.com",
    #         "password": "password",
    #         "confirm": "password",
    #     }
    #     response = test_client.post(
    #         f"{module_info['url_prefix']}/register",
    #         data=data,
    #         follow_redirects=True,
    #     )
    #     user = User.query.filter(User.email == "test@gmail.com").scalar()

    #     assert response.status_code == 200
    #     assert request.path == url_for('auth.register')
    #     assert user.is_email_confirmed is True

    # @pytest.mark.parametrize(
    #     "email_config",
    #     [
    #         ("EMAIL_CONFIRMATION_DISABLED", "remove"),
    #         ("EMAIL_CONFIRMATION_DISABLED", False),
    #         ("EMAIL_CONFIRMATION_DISABLED", None),
    #     ],
    #     indirect=True,
    # )
    # def test_user_is_registered_on_valid_form_submit(
    #     self, test_client, capfd, email_config
    # ):
    #     data = {
    #         "email": "test@gmail.com",
    #         "password": "password",
    #         "confirm": "password",
    #     }
    #     response = test_client.post(
    #         f"{module_info['url_prefix']}/register",
    #         data=data,
    #         follow_redirects=True,
    #     )
    #     # Not very happy with this solution. Need a better
    #     # way to wait for the email thread to join with main
    #     # thread before reading the email written to stdout @rehmanis
    #     while threading.activeCount() > 1:
    #         pass
    #     else:
    #         captured = capfd.readouterr()

    #     user = User.query.filter(User.email == "test@gmail.com").scalar()

    #     assert response.status_code == 200
    #     assert request.path == url_for("auth.unconfirmed")
    #     assert b"A confirmation email has been sent via email" in response.data
    #     assert "test@gmail.com" in captured.out
    #     assert "Welcome to Shopyo" in captured.out
    #     assert user is not None
    #     assert user.is_email_confirmed is False

    # @pytest.mark.usefixtures("login_non_admin_user")
    # def test_user_not_confirmed_for_already_confirmed_user(self, test_client):
    #     response = test_client.get(
    #         url_for("auth.confirm", token="sometoken"), follow_redirects=True
    #     )

    #     assert response.status_code == 200
    #     assert request.path == url_for("dashboard.index")
    #     assert b"Account already confirmed." in response.data

    # @pytest.mark.usefixtures("login_unconfirmed_user")
    # def test_user_confirmed_on_valid_token(self, test_client, current_user):
    #     token = current_user.generate_confirmation_token()
    #     response = test_client.get(
    #         url_for("auth.confirm", token=token), follow_redirects=True
    #     )

    #     assert response.status_code == 200
    #     # assert request.path == url_for("dashboard.index")
    #     # assert b"You have confirmed your account. Thanks!" in response.data
    #     assert current_user.is_email_confirmed is True

    # @pytest.mark.usefixtures("login_unconfirmed_user")
    # def test_no_confirm_sent_for_invalid_token(self, test_client, current_user):
    #     token = current_user.generate_confirmation_token() + "extra"
    #     response = test_client.get(
    #         url_for("auth.confirm", token=token), follow_redirects=True
    #     )

    #     assert response.status_code == 200
    #     assert request.path == url_for("auth.unconfirmed")
    #     assert b"The confirmation link is invalid/expired." in response.data

    # @pytest.mark.usefixtures("login_non_admin_user")
    # def test_do_not_allow_email_resend_for_confirmed(self, test_client):
    #     response = test_client.get(url_for("auth.resend"), follow_redirects=True)

    #     assert response.status_code == 200
    #     assert request.path == '/'

    # @pytest.mark.usefixtures("login_unconfirmed_user")
    # def test_valid_resend_email_confirmation(self, test_client, capfd, current_user):
    #     response = test_client.get(url_for("auth.resend"), follow_redirects=True)

    #     # Not very happy with this solution. Need a better
    #     # way to wait for the email thread to join with main
    #     # thread before reading the email written to stdout @rehmanis
    #     while threading.activeCount() > 1:
    #         pass
    #     else:
    #         captured = capfd.readouterr()

    #     assert response.status_code == 200
    #     assert current_user.email in captured.out
    #     assert "Welcome to Shopyo" in captured.out
    #     assert request.path == url_for("auth.unconfirmed")
    #     assert b"A new confirmation email has been sent" in response.data

    # @pytest.mark.usefixtures("login_non_admin_user")
    # def test_confirmed_user_is_redirected_to_dashboard(self, test_client):
    #     response = test_client.get(url_for("auth.unconfirmed"), follow_redirects=True)

    #     assert response.status_code == 200
    #     assert request.path == '/'

    # @pytest.mark.usefixtures("login_unconfirmed_user")
    # def test_unconfirmed_page_renders_correctly(self, test_client, current_user):
    #     response = test_client.get(url_for("auth.unconfirmed"))

    #     assert response.status_code == 302
    #     assert request.path == url_for("auth.unconfirmed")
    #     assert b"You have not confirmed your account" in response.data
    #     assert b"Email confirmation link was sent to" in response.data
    #     assert current_user.email.encode() in response.data

    # def test_login_for_dashboard_renders(self, test_client):
    #     response = test_client.get(url_for("auth.login"))

    #     assert response.status_code == 200
    #     assert b"Login" in response.data
    #     assert b"submit" in response.data

    # def test_invalid_dashboard_login(self, test_client):
    #     response = test_client.post(
    #         url_for("auth.login"),
    #         data=dict(email="admin1@domain.com", password="wrongpass"),
    #         follow_redirects=True,
    #     )

    #     assert response.status_code == 200
    #     assert request.path == url_for("auth.login")
    #     assert b"please check your user id and password" in response.data

    # def test_valid_dashboard_login(self, test_client, admin_user, current_user):
    #     response = test_client.post(
    #         url_for("auth.login"),
    #         data=dict(email=admin_user.email, password="pass"),
    #         follow_redirects=True,
    #     )

    #     assert response.status_code == 200
    #     assert current_user.email == admin_user.email
    #     assert request.path == url_for("dashboard.index")

    # def test_valid_dashboard_login_is_case_insensitive(self, test_client, current_user):
    #     try:
    #         User.create(email="foo@bar.com", password="pass")
    #     except sqlalchemy.exc.IntegrityError:
    #         pass
    #     data = {"email": "Foo@Bar.com", "password": "pass"}
    #     response = test_client.post(
    #         url_for("auth.login"),
    #         data=data,
    #         follow_redirects=True,
    #     )

    #     assert response.status_code == 200
    #     assert current_user.email.lower() == data["email"].lower()
    #     assert request.path == url_for("auth.unconfirmed")

    # @pytest.mark.usefixtures("login_non_admin_user")
    # def test_current_user_logout(self, test_client, current_user):
    #     response = test_client.get(url_for("auth.logout"), follow_redirects=True)

    #     assert response.status_code == 200
    #     assert request.path == url_for("auth.login")
    #     assert b"Successfully logged out" in response.data
    #     assert current_user.is_authenticated is False



"""Factories to help in tests."""
from factory import Sequence
from factory.alchemy import SQLAlchemyModelFactory
from init import db
from modules.box__default.auth.models import Role
from modules.box__default.auth.models import User
from sqlalchemy.orm import scoped_session


class BaseFactory(SQLAlchemyModelFactory):
    """Base factory."""

    class Meta:
        """Factory configuration."""

        abstract = True
        sqlalchemy_session = db.session
        sqlalchemy_session = scoped_session(
            lambda: db.session, scopefunc=lambda: db.session
        )
        sqlalchemy_session_persistence = "commit"


class UserFactory(BaseFactory):
    """User factory."""

    username = Sequence(lambda n: f"user{n}")
    email = Sequence(lambda n: f"user{n}@example.com")
    password = "pass"
    is_email_confirmed = True
    is_admin = False

    class Meta:
        """Factory configuration."""

        model = User


class RoleFactory(BaseFactory):
    """Role factory."""

    name = Sequence(lambda n: f"role{n}")

    class Meta:
        """Factory configuration."""

        model = Role



# import os
# import pytest
# from shopyo import __main__
# def test_no_args(monkeypatch, capfd):
#      monkeypatch.setattr("sys.argv", [""])
#      __main__.main()
#      captured = capfd.readouterr()
#      assert "No arguments supplied" in captured.out
# def test_arg_no_env(monkeypatch, capfd):
#      monkeypatch.setattr("sys.argv", ["testok"])
#      __main__.main()
#      captured = capfd.readouterr()
#      assert "Please use Shopyo in a virtual environment for this command" in captured.out
import subprocess
import sys


def test_no_args(capfd):
    subprocess.run([sys.executable, "__main__.py"], text=True)
    captured = capfd.readouterr()
    assert "No arguments supplied" in captured.out



import os

import pytest
from click.testing import CliRunner

from shopyo.api.cli import cli
from shopyo.api.constants import SEP_CHAR
from shopyo.api.constants import SEP_NUM

pytestmark = pytest.mark.cli_unit


@pytest.fixture(scope="session")
def cli_runner():
    """Fixture that returns a helper function to run the shopyo cli."""
    runner = CliRunner()

    def cli_main(*cli_args, **cli_kwargs):
        """Run shopyo cli main with the given args."""
        return runner.invoke(cli, cli_args, **cli_kwargs)

    return cli_main


@pytest.mark.usefixtures("restore_cwd")
class TestCliCreateBox:
    """test the create_box command line api function"""

    def test_create_existing_box(self, tmpdir, cli_runner):
        tmpdir.mkdir("modules").mkdir("box__foo")
        os.chdir(tmpdir)
        module_path = os.path.join("modules", "box__foo")
        result = cli_runner("startbox", "box__foo")
        expected = f"[ ] unable to create. Box {module_path} already exists!"

        assert result.exit_code != 0
        assert expected in result.output

    @pytest.mark.parametrize("opt", ["-v", "--verbose"])
    def test_create_unique_box(self, tmpdir, cli_runner, opt):
        tmpdir.mkdir("modules")
        os.chdir(tmpdir)
        result = cli_runner("startbox", "box__foo", opt)
        module_path = os.path.join("modules", "box__foo")
        expected = f"[x] Successfully created dir {module_path}"

        assert result.exit_code == 0
        assert os.path.exists(os.path.join("modules", "box__foo"))
        assert os.path.exists(os.path.join("modules", "box__foo", "box_info.json"))
        assert expected in result.output


@pytest.mark.usefixtures("restore_cwd")
@pytest.mark.order("last")
class TestCliClean:
    """tests the clean command line api function"""

    def test_clean_pycache_present_only_in_cwd(self, tmpdir, flask_app):
        """
        run `shopyo clean2 -v` on the following test directory:

        <some-unique-tmpdir>/
            __pycache__/
                file.pyc
        """
        fd = tmpdir.mkdir("__pycache__").join("file.pyc")
        fd.write("content")
        os.chdir(tmpdir)
        runner = flask_app.test_cli_runner(mix_stderr=False)
        result = runner.invoke(cli, ["clean", "-v"])
        expected_out = "[x] all tables dropped\n[x] __pycache__ successfully deleted\n"
        expected_err_shopyo_db = (
            f"[ ] unable to delete {os.path.join(tmpdir, 'shopyo.db')}"
        )
        expected_err_migrations = (
            f"[ ] unable to delete {os.path.join(tmpdir, 'migrations')}"
        )

        assert result.exit_code == 0
        assert os.path.exists("__pycache__") is False
        assert expected_out in result.output
        assert expected_err_shopyo_db in result.stderr
        assert expected_err_migrations in result.stderr

    def test_clean_pycache_many_lvls_below_cwd(self, tmpdir, flask_app):
        """
        run `shopyo clean2 -v` on the following test directory:

        <some-unique-tmpdir>/
            shopyo/
                shopyo/
                    mod/
                        box/
                        __pycache__/
                            file.pyc
        """

        path = tmpdir.mkdir("shopyo").mkdir("shopyo").mkdir("mod").mkdir("box")
        pycache_path = path.mkdir("__pycache__")
        pyc = pycache_path.join("file.pyc")
        pyc.write("content")
        os.chdir(tmpdir)
        runner = flask_app.test_cli_runner()
        result = runner.invoke(cli, ["clean", "-v"])
        expected_out = "[x] all tables dropped\n[x] __pycache__ successfully deleted\n"
        expected_err_shopyo_db = (
            f"[ ] unable to delete {os.path.join(tmpdir, 'shopyo.db')}"
        )
        expected_err_migrations = (
            f"[ ] unable to delete {os.path.join(tmpdir, 'migrations')}"
        )

        assert result.exit_code == 0
        assert os.path.exists(pycache_path) is False
        assert expected_out in result.output
        assert expected_err_shopyo_db in result.output
        assert expected_err_migrations in result.output

    def test_clean_many_pycache_in_nested_dirs(self, tmpdir, flask_app):
        """
        run `shopyo clean2 -v` on the following test directory:

        <some-unique-tmpdir>/
            __pycache__/
                file.pyc
            shopyo/
                __pycache__
                    file.pyc
                module/
                    __pycache__
                        file.pyc
        """
        pycache_path1 = tmpdir.mkdir("__pycache__")
        pyc1 = pycache_path1.join("file.pyc")
        pyc1.write("content")
        shopyo_path = tmpdir.mkdir("shopyo")
        pycache_path2 = shopyo_path.mkdir("__pycache__")
        pyc2 = pycache_path2.join("file.pyc")
        pyc2.write("content")
        pycache_path3 = shopyo_path.mkdir("module").mkdir("__pycache__")
        pyc3 = pycache_path3.join("file.pyc")
        pyc3.write("content")
        os.chdir(tmpdir)
        runner = flask_app.test_cli_runner()
        result = runner.invoke(cli, ["clean", "-v"])
        expected_out = "[x] all tables dropped\n[x] __pycache__ successfully deleted\n"
        expected_err_shopyo_db = (
            f"[ ] unable to delete {os.path.join(tmpdir, 'shopyo.db')}"
        )
        expected_err_migrations = (
            f"[ ] unable to delete {os.path.join(tmpdir, 'migrations')}"
        )

        assert result.exit_code == 0
        assert os.path.exists(pycache_path1) is False
        assert os.path.exists(pycache_path2) is False
        assert os.path.exists(pycache_path3) is False
        assert expected_out in result.output
        assert expected_err_shopyo_db in result.output
        assert expected_err_migrations in result.output

    def test_no_clean_applied_on_multiple_pycache(self, tmpdir):
        """
        run no clean command on the following test directory:

        <some-unique-tmpdir>/
                __pycache__/
                shopyo/
                    __pycache__/

        """
        path1 = tmpdir.mkdir("__pycache__")
        path2 = tmpdir.mkdir("shopyo").mkdir("__pycache__")

        assert os.path.exists(path1)
        assert os.path.exists(path2)

    def test_clean_on_skip_shopyo_db_file(self, tmpdir, flask_app):
        """
        run `shopyo clean2 -v` on the following test directory:

        <some-unique-tmpdir>/
            shopyo.db
        """
        shopyo_db = tmpdir.join("shopyo.db")
        shopyo_db.write("content")
        os.chdir(tmpdir)
        runner = flask_app.test_cli_runner()
        result = runner.invoke(cli, ["clean", "--no-clear-db", "-v"])
        part_expected_out = "[ ] db clearing skipped"

        assert result.exit_code == 0
        assert os.path.exists(shopyo_db) is True
        assert part_expected_out in result.output

    def test_clean_on_shopyo_db_file(self, tmpdir, flask_app):
        """
        run `shopyo clean2 -v` on the following test directory:

        <some-unique-tmpdir>/
            shopyo.db
        """
        shopyo_db = tmpdir.join("shopyo.db")
        shopyo_db.write("content")
        os.chdir(tmpdir)
        runner = flask_app.test_cli_runner()
        result = runner.invoke(cli, ["clean", "-v"])
        expected_out1 = "[x] all tables dropped"
        expected_out2 = "[ ] __pycache__ doesn't exist"
        expected_out3 = (
            f"[x] file '{os.path.join(tmpdir, 'shopyo.db')}' successfully deleted"
        )
        expected_out4 = f"[ ] unable to delete {os.path.join(tmpdir, 'migrations')}"

        assert result.exit_code == 0
        assert os.path.exists(shopyo_db) is False
        assert expected_out1 in result.output
        assert expected_out2 in result.output
        assert expected_out3 in result.output
        assert expected_out4 in result.output

    def test_clean_on_skip_migration_folder(self, tmpdir, flask_app):
        """
        run `shopyo clean2 -v` on the following test directory:

        <some-unique-tmpdir>/
            migrations/
                env.py
                alembic.ini
        """
        migrations_path = tmpdir.mkdir("migrations")
        env = migrations_path.join("env.py")
        alembic = migrations_path.join("alembic.ini")
        env.write("content-env")
        alembic.write("content-alembic")
        os.chdir(tmpdir)
        runner = flask_app.test_cli_runner(mix_stderr=False)
        result = runner.invoke(cli, ["clean", "--no-clear-migration", "-v"])
        part_expected_out = "[ ] migration folder delete skipped"

        assert result.exit_code == 0
        assert os.path.exists(migrations_path) is True
        assert part_expected_out in result.stdout

    def test_clean_on_migration_folder(self, tmpdir, flask_app):
        """
        run `shopyo clean2 -v` on the following test directory:

        <some-unique-tmpdir>/
            migrations/
                env.py
                alembic.ini
        """
        migrations_path = tmpdir.mkdir("migrations")
        env = migrations_path.join("env.py")
        alembic = migrations_path.join("alembic.ini")
        env.write("content-env")
        alembic.write("content-alembic")
        os.chdir(tmpdir)
        runner = flask_app.test_cli_runner(mix_stderr=False)
        result = runner.invoke(cli, ["clean", "-v"])
        expected_out = (
            "[x] all tables dropped\n"
            f"[x] folder '{os.path.join(tmpdir, 'migrations')}' "
            "successfully deleted\n"
        )
        expected_err_pycache = "[ ] __pycache__ doesn't exist\n"
        expected_err_shopyo_db = (
            f"[ ] unable to delete {os.path.join(tmpdir, 'shopyo.db')}"
        )

        assert result.exit_code == 0
        assert os.path.exists(migrations_path) is False
        assert expected_out in result.stdout
        assert expected_err_pycache in result.stderr
        assert expected_err_shopyo_db in result.stderr

    @pytest.mark.parametrize("option", ["-v", "--verbose"])
    def test_clean_pycache_shopyo_migration(self, tmpdir, flask_app, option):
        """
        run `shopyo clean2 -v` on the following test directory

        shopyo/
            shopyo/
                migrations/
                    alembic.ini
                    env.py
                module1/
                    __pycache__/
                        file.pyc
                module2/
                    __pycache__/
                        file.pyc
                shopyo.db
        """
        shopyo_path = tmpdir.mkdir("shopyo").mkdir("shopyo")
        migrations_path = shopyo_path.mkdir("migrations")
        env = migrations_path.join("env.py")
        env.write("content-env")
        alembic = migrations_path.join("alembic.ini")
        alembic.write("content-alembic")
        pycache_path1 = shopyo_path.mkdir("module1").mkdir("__pycache__")
        pycache_path2 = shopyo_path.mkdir("module2").mkdir("__pycache__")
        pyc1 = pycache_path1.join("file.pyc")
        pyc1.write("content")
        pyc2 = pycache_path2.join("file.pyc")
        pyc2.write("content")
        shopyo_db = shopyo_path.join("shopyo.db")
        shopyo_db.write("content")
        os.chdir(shopyo_path)
        runner = flask_app.test_cli_runner()
        result = runner.invoke(cli, ["clean", option])
        expected_out1 = "[x] all tables dropped"
        expected_out2 = "[x] __pycache__ successfully deleted"
        expected_out3 = (
            f"[x] file '{os.path.join(shopyo_path, 'shopyo.db')}' successfully deleted"
        )
        expected_out4 = (
            f"[x] folder '{os.path.join(shopyo_path, 'migrations')}' successfully"
            " deleted\n"
        )

        assert result.exit_code == 0
        assert expected_out1 in result.output
        assert expected_out2 in result.output
        assert expected_out3 in result.output
        assert expected_out4 in result.output
        assert os.path.exists(migrations_path) is False
        assert os.path.exists(pycache_path1) is False
        assert os.path.exists(pycache_path2) is False
        assert os.path.exists(shopyo_db) is False

    def test_no_clean_on_shopyo_and_migrations(self, tmpdir):
        """
        run test on the following test directory:

        <some-unique-tmpdir>/
            migrations/
            shopyo.db
        """
        migrations_path = tmpdir.mkdir("migrations")
        shopyo_db = tmpdir.join("shopyo.db")
        shopyo_db.write("content")

        assert os.path.exists(migrations_path)
        assert os.path.exists(shopyo_db)

    @pytest.mark.parametrize("option", ["-v", "--verbose"])
    def test_clean_on_no_files_to_clean(self, tmpdir, flask_app, option):
        os.chdir(tmpdir)
        runner = flask_app.test_cli_runner(mix_stderr=False)
        result = runner.invoke(cli, ["clean", option])
        expected_out = "[x] all tables dropped\n"
        expected_err_pycache = "[ ] __pycache__ doesn't exist\n"
        expected_err_shopyo_db = (
            f"[ ] unable to delete {os.path.join(tmpdir, 'shopyo.db')}"
        )
        expected_err_migrations = (
            f"[ ] unable to delete {os.path.join(tmpdir, 'migrations')}"
        )

        assert result.exit_code == 0
        assert expected_out in result.output
        assert expected_err_pycache in result.stderr
        assert expected_err_shopyo_db in result.stderr
        assert expected_err_migrations in result.stderr

    def test_clean_with_no_verbose_on_empty_dir(self, tmpdir, flask_app):
        """
        run `shopyo clean2` on empty directory
        """
        os.chdir(tmpdir)
        runner = flask_app.test_cli_runner(mix_stderr=False)
        result = runner.invoke(cli, ["clean"])
        expect_out = "Cleaning...\n" + SEP_CHAR * SEP_NUM + "\n"

        assert result.exit_code == 0
        assert expect_out in result.output

    def test_clean_with_no_verbose_on_all_files(self, tmpdir, flask_app):
        """
        run `shopyo clean2` in directory with all files(shopyo.db,
        migrations/, __pycahce__/)
        """
        pycache_path = tmpdir.mkdir("__pycache__")
        shopyo_path = tmpdir.join("shopyo.db")
        shopyo_path.write("content")
        migrations_path = tmpdir.mkdir("migrations")
        os.chdir(tmpdir)
        runner = flask_app.test_cli_runner(mix_stderr=False)
        result = runner.invoke(cli, ["clean"])
        expect_out1 = "Cleaning...\n" + SEP_CHAR * SEP_NUM + "\n"
        expect_out2 = "[x] __pycache__ successfully deleted\n"
        expect_out3 = "[ ] unable to delete"

        assert result.exit_code == 0
        assert expect_out1 in result.output
        assert expect_out2 not in result.output
        assert expect_out3 not in result.output
        assert os.path.exists(pycache_path) is False
        assert os.path.exists(shopyo_path) is False
        assert os.path.exists(migrations_path) is False

    # TODO: add test_clean for MySQL to see if tables dropped @rehmanis


@pytest.mark.usefixtures("restore_cwd")
class TestCliNew:
    @pytest.mark.parametrize("proj,parent", [("", "foo"), ("bar", "")])
    def test_new_projname_already_exists(self, cli_runner, proj, parent, tmp_path):
        name = proj or parent
        parent_path = tmp_path / parent
        proj_path = parent_path / name
        proj_path.mkdir(parents=True)
        os.chdir(parent_path)
        result = cli_runner("new", proj)
        expected_out = (
            f"[ ] Error: Unable to create new project. Path {proj_path} exits"
        )

        assert result.exit_code == 1
        assert expected_out in result.output

    def test_new_project_invvalid_projname(self, cli_runner):
        result = cli_runner("new", ")foo?")
        expected_out = (
            "[ ] Error: PROJNAME is not valid, please use alphanumeric "
            "and underscore only"
        )

        assert result.exit_code == 1
        assert expected_out in result.output

    @pytest.mark.parametrize("proj,parent", [("", "foo"), ("bar", "")])
    def test_new_project_valid_name(self, cli_runner, tmp_path, proj, parent):
        # create the parent folder for foo and none for bar proj
        temp_proj = tmp_path / parent
        # print(temp_proj)
        temp_proj.mkdir(exist_ok=True)
        # change to the parent folder (in case of bar it tmp_path while for foo
        # it is tmp_path/foo) and run the new command. For foo, there is no
        # argument for proj provided so it will create a foo/ project inside
        # tmp_path/foo while for bar it will create bar/bar/ inside tmp_path
        os.chdir(temp_proj)
        result = cli_runner("new", proj)
        # change back to tmp_path so that for easier comparision
        os.chdir(tmp_path)
        # use this for the name of the project that was created in tmp_path
        name = parent or proj

        assert result.exit_code == 0
        assert os.path.exists(os.path.join(name, name))
        assert os.path.exists(os.path.join(name, "requirements.txt"))
        assert os.path.exists(os.path.join(name, "dev_requirements.txt"))
        assert os.path.exists(os.path.join(name, "tox.ini"))
        assert os.path.exists(os.path.join(name, "MANIFEST.in"))
        assert os.path.exists(os.path.join(name, "README.md"))
        assert os.path.exists(os.path.join(name, ".gitignore"))
        assert os.path.exists(os.path.join(name, "pytest.ini"))
        assert os.path.exists(os.path.join(name, "setup.py"))
        assert os.path.exists(os.path.join(name, name, "__init__.py"))
        assert os.path.exists(os.path.join(name, name, "cli.py"))
        assert os.path.exists(os.path.join(name, "docs"))
        assert os.path.exists(os.path.join(name, "docs", "conf.py"))
        assert os.path.exists(os.path.join(name, "docs", "_static"))
        assert os.path.exists(os.path.join(name, "docs", "_static", "custom.css"))
        assert os.path.exists(os.path.join(name, "docs", "Makefile"))
        assert os.path.exists(os.path.join(name, "docs", "index.rst"))
        assert os.path.exists(os.path.join(name, "docs", "docs.rst"))
        assert not os.path.exists(os.path.join(name, name, "__main__.py"))
        assert not os.path.exists(os.path.join(name, name, "api"))
        assert not os.path.exists(os.path.join(name, name, ".tox"))
        assert not os.path.exists(os.path.join(name, name, ".coverage"))
        assert not os.path.exists(os.path.join(name, name, "shopyo.db"))
        assert not os.path.exists(os.path.join(name, name, "testing.db"))
        assert not os.path.exists(os.path.join(name, name, "coverage.xml"))
        assert not os.path.exists(os.path.join(name, name, "setup.cfg"))
        assert not os.path.exists(os.path.join(name, name, "instance"))
        assert not os.path.exists(os.path.join(name, name, "migrations"))
        assert not os.path.exists(os.path.join(name, name, "__pycache__"))
        assert not os.path.exists(os.path.join(name, name, "config.json"))


class TestCliInitialise:
    pass


@pytest.mark.usefixtures("fake_foo_proj")
class TestCliStartapp:
    @pytest.mark.parametrize("mod", ["box_bar", "box__foo"])
    def test_create_invalid_modulename_with_box_prefix(self, cli_runner, mod):
        result = cli_runner("startapp", mod)
        expected_out = (
            f"[ ] Invalid MODULENAME '{mod}'. MODULENAME cannot start"
            " with box_ prefix\n"
        )

        assert result.exit_code != 0
        assert expected_out in result.output

    @pytest.mark.parametrize("box", ["box_bar", "boxfoo", "foo"])
    def test_create_module_with_invalid_box_name(self, cli_runner, box):
        result = cli_runner("startapp", "demo", box)
        expected_out = (
            f"[ ] Invalid BOXNAME '{box}'. BOXNAME should start with 'box__' prefix\n"
        )

        assert result.exit_code != 0
        assert expected_out in result.output

    @pytest.mark.parametrize(
        "mod,box",
        [
            ("bar", ""),
            ("foo", "box__default"),
            ("foo", ""),
            ("baz", "box__default"),
            ("demo", "box__default"),
        ],
    )
    def test_create_existing_module(self, cli_runner, mod, box, tmpdir):
        result = cli_runner("startapp", mod, box)
        expected_out = (
            f"[ ] Unable to create module '{mod}'. "
            "MODULENAME already exists inside modules/ at"
        )

        assert result.exit_code != 0
        assert expected_out in result.output

    def test_create_modulename_not_alphanumeric(self, cli_runner):
        result = cli_runner("startapp", "my(demo)mod")
        expected_out = (
            "[ ] Error: MODULENAME is not valid, please use alphanumeric "
            "and underscore only\n"
        )

        assert result.exit_code != 0
        assert expected_out in result.output

    def test_create_boxname_not_alphanumeric(self, cli_runner):
        result = cli_runner("startapp", "mod", "box__?.game")
        expected_out = (
            "[ ] Error: BOXNAME is not valid, please use alphanumeric "
            "and underscore only\n"
        )

        assert result.exit_code != 0
        assert expected_out in result.output

    @pytest.mark.parametrize(
        "mod,box",
        [
            ("store", ""),  # create module
            ("store", "box__ecommerce"),  # create submodule and also box
            ("marketplace", "box__default"),  # create submodule but not box
        ],
    )
    def test_create_valid_modules(self, cli_runner, fake_foo_proj, mod, box):
        result = cli_runner("startapp", mod, box)
        module_path = os.path.join(fake_foo_proj, "modules", box, mod)

        assert result.exit_code == 0
        assert os.path.exists(module_path)
        assert os.path.exists(os.path.join(module_path, "templates"))
        assert os.path.exists(os.path.join(module_path, "templates", mod))
        assert os.path.exists(
            os.path.join(module_path, "templates", mod, "blocks", "sidebar.html")
        )
        assert os.path.exists(
            os.path.join(module_path, "templates", mod, "dashboard.html")
        )
        assert os.path.exists(os.path.join(module_path, "tests"))
        assert os.path.exists(os.path.join(module_path, "static"))
        assert os.path.exists(
            os.path.join(module_path, "tests", f"test_{mod}_functional.py")
        )
        assert os.path.exists(
            os.path.join(module_path, "tests", f"test_{mod}_models.py")
        )
        assert os.path.exists(os.path.join(module_path, "view.py"))
        assert os.path.exists(os.path.join(module_path, "forms.py"))
        assert os.path.exists(os.path.join(module_path, "models.py"))
        assert os.path.exists(os.path.join(module_path, "info.json"))
        assert os.path.exists(os.path.join(module_path, "global.py"))

    @pytest.mark.parametrize("opt", ["-v", "--verbose"])
    def test_create_valid_module_with_verbose(self, cli_runner, fake_foo_proj, opt):
        result = cli_runner("startapp", "store", opt)
        module_path = os.path.join(fake_foo_proj, "modules", "store")
        expected_out1 = "[x] Successfully created"
        expected_out2 = "created with content"

        assert result.exit_code == 0
        assert os.path.exists(module_path)
        assert expected_out1 in result.output
        assert expected_out2 in result.output


@pytest.mark.usefixtures("fake_foo_proj")
class TestCliCollectstatic:
    def test_collectstatic_with_default_src(self, cli_runner):
        result = cli_runner("collectstatic")
        expected_out1 = "Collecting static...\n" + SEP_CHAR * SEP_NUM + "\n\n"
        expected_out2 = "[x] done copying"

        assert result.exit_code == 0
        assert expected_out1 in result.output
        assert expected_out2 not in result.output
        assert os.path.exists("static/modules/bar/bar.css")
        assert os.path.exists("static/modules/box__default/foo/foo.css")
        assert len(os.listdir("static/modules")) == 2
        assert len(os.listdir("static/modules/box__default")) == 2

    @pytest.mark.parametrize(
        "src,expected",
        [
            ("box__default", "box__default/foo/foo.css"),
            ("box__default", "box__default/zoo/zoo.css"),
            ("box__default/foo", "box__default/foo/foo.css"),
            ("bar", "bar/bar.css"),
            ("box__default\\foo", "box__default/foo/foo.css"),
            ("modules/bar", "bar/bar.css"),
            ("modules\\box__default/foo", "box__default/foo/foo.css"),
        ],
    )
    def test_collectstatic_with_valid_arg(self, src, expected, cli_runner):
        result = cli_runner("collectstatic", src)
        expected_out1 = "Collecting static...\n" + SEP_CHAR * SEP_NUM + "\n\n"
        expected_out2 = "[x] done copying"

        assert result.exit_code == 0
        assert expected_out1 in result.output
        assert expected_out2 not in result.output
        assert os.path.exists(f"static/modules/{expected}")
        assert len(os.listdir("static/modules")) == 1

    def test_collectstatic_with_invalid_arg(self, cli_runner):
        result = cli_runner("collectstatic", "foobar")
        modules_path = os.path.join("modules", "foobar")
        modules_path = os.path.join(os.getcwd(), modules_path)
        expected_out = f"[ ] path: {modules_path} does not exist"

        assert result.exit_code != 0
        assert expected_out in result.output
        assert not os.path.exists("static/modules")

    @pytest.mark.parametrize("option", ["-v", "--verbose"])
    def test_collectstatic_with_verbose(self, cli_runner, option):
        result = cli_runner("collectstatic", option)
        expected_out1 = "Collecting static...\n" + SEP_CHAR * SEP_NUM + "\n"
        expected_out2 = "[x] done copying"

        assert result.exit_code == 0
        assert expected_out1 in result.output
        assert expected_out2 in result.output



from functools import wraps

from flask import redirect
from flask_login import current_user
from init import login_manager
from modules.box__default.auth.models import User

from shopyo.api.html import notify_warning

login_manager.login_view = "auth.login"
login_manager.login_message = notify_warning("Please login for access")


@login_manager.user_loader
def load_user(id):
    return User.query.get(id)


def admin_required(f):
    @wraps(f)
    def wrap(*args, **kwargs):
        if current_user.is_admin:
            return f(*args, **kwargs)
        else:
            return redirect("/")

    return wrap



import subprocess
import sys


def test_managepy(capfd):
    subprocess.run([sys.executable, "manage.py", "testok"], text=True)
    captured = capfd.readouterr()
    assert "test ok!" in captured.out



from flask import render_template


def yo_render(template, context_dict):
    """
    Renders template.

    Usage: yo_render("index.html", {"x": 1, "y": 2})
    Same as render_template("index.html", x=1, y=2)


    Args:
        template (String): template accessing
        context_dict (Dict): Template values

    Returns
    -------
    html of template
    """
    return render_template(template, **context_dict)



from init import db


class Settings(db.Model):
    __tablename__ = "settings"
    setting = db.Column(db.String(100), primary_key=True)
    value = db.Column(db.String(100))

    def add(self):
        db.session.add(self)

    def insert(self):
        db.session.add(self)
        db.session.commit()

    def update(self):
        db.session.commit()

    def delete(self):
        db.session.delete(self)
        db.session.commit()



import datetime

from shopyo.api.info import printinfo


def test_no_args(capfd):
    printinfo()
    datetime.datetime.now().year
    captured = capfd.readouterr()
    assert (
        r"""
  / ____| |
 | (___ | |__   ___  _ __  _   _  ___
  \___ \| '_ \ / _ \| '_ \| | | |/ _ \
  ____) | | | | (_) | |_) | |_| | (_) |
 |_____/|_| |_|\___/| .__/ \__, |\___/
                    | |     __/ |
                    |_|    |___/
Copyright {year}""".format(
            year=datetime.datetime.now().year
        )
        in captured.out
    )



import copy
import json
import os

from flask import Blueprint
from flask import redirect
from flask import render_template
from flask import url_for

from shopyo.api.assets import get_static


class ModuleHelp:
    def __init__(self, dunderfile, dundername):
        self.dirpath = os.path.dirname(os.path.abspath(dunderfile))
        self.template_folder = os.path.join(self.dirpath, "templates")
        self.info = {}
        self._context = {}

        with open(self.dirpath + "/info.json") as f:
            self.info = json.load(f)

        self.blueprint_str = "{}_blueprint".format(self.info["module_name"])
        self.blueprint = Blueprint(
            "{}".format(self.info["module_name"]),
            dundername,
            template_folder=self.template_folder,
            url_prefix=self.info["url_prefix"],
        )

        self._context.update({"info": self.info})

    def render(self, filename, **kwargs):
        """
        renders file.html found in module/templates/module/file.html
        """
        return render_template(
            "{}/{}".format(self.info["module_name"], filename), **kwargs
        )

    def redirect_url(self, url, **kwargs):
        return redirect(url_for(url, **kwargs))

    def context(self):
        return copy.deepcopy(self._context)

    def method(self, methodname):
        return "{}.{}".format(self.info["module_name"], methodname)

    def get_self_static(self, filename):
        module_parent = os.path.dirname(self.dirpath)
        module_folder = self.dirpath

        module_parent = os.path.normpath(module_parent)
        module_parent = os.path.basename(module_parent)

        module_folder = os.path.normpath(module_folder)
        module_folder = os.path.basename(module_folder)

        print(module_parent, module_parent)
        if module_parent.startswith("box__"):
            boxormodule = f"{module_parent}/{module_folder}"
        else:
            boxormodule = module_folder
        return get_static(boxormodule=boxormodule, filename=filename)



# global templates variables in here
from .helpers import get_current_lang
from .helpers import get_default_lang
from .helpers import langs

available_everywhere = {
    "get_i18n_langs": langs,
    "get_default_lang": get_default_lang,
    "get_current_lang": get_current_lang,
}
# global configs in here, defined by profile
# configs = {
#     "development": {
#         "CONFIG_VAR": "DEVVALUE"
#     },
#     "production": {
#         "CONFIG_VAR": "PRODVALUE"
#     },
#     "testing": {
#         "CONFIG_VAR": "TESTVALUE"
#     }
# }



"""
All initializations like db = SQLAlchemy in this file
"""
import os

from flask_login import LoginManager
from flask_mailman import Mail
from flask_migrate import Migrate
from flask_sqlalchemy import SQLAlchemy
from flask_wtf.csrf import CSRFProtect

# from flask_marshmallow import Marshmallow, uncommented as not updated to support flask 2.x

root_path = os.path.dirname(os.path.abspath(__file__))  # don't remove
static_path = os.path.join(root_path, "static")  # don't remove
modules_path = os.path.join(root_path, "modules")  # don't remove
themes_path = os.path.join(static_path, "themes")  # don't remove
installed_packages = []  # don't remove

installed_packages = []

db = SQLAlchemy()
# ma = Marshmallow()
login_manager = LoginManager()
migrate = Migrate()
mail = Mail()
csrf = CSRFProtect()


def load_extensions(app):
    migrate.init_app(app, db)
    db.init_app(app)
    # ma.init_app(app)
    mail.init_app(app)
    login_manager.init_app(app)
    csrf.init_app(app)



from flask import redirect
from flask import render_template
from flask import request
from flask import url_for
from flask_login import login_required
from init import db
from modules.box__default.i18n.helpers import get_current_lang

from .forms import PageForm
from .models import Page
from shopyo.api.forms import flash_errors
from shopyo.api.module import ModuleHelp

mhelp = ModuleHelp(__file__, __name__)
globals()[mhelp.blueprint_str] = mhelp.blueprint
module_blueprint = globals()[mhelp.blueprint_str]


module_name = mhelp.info["module_name"]

sidebar = [{"text": "sample", "icon": "fa fa-table", "url": ""}]

module_settings = {"sidebar": sidebar}


@module_blueprint.route(mhelp.info["dashboard"] + "/all")
def index_all():
    context = {}
    pages = Page.query.all()

    context.update({"pages": pages})
    return render_template("page/all_pages.html", **context)


@module_blueprint.route(mhelp.info["dashboard"] + "/all-pages")
def index():
    context = {}
    pages = Page.query.all()

    context.update({"pages": pages})
    return render_template("page/all_pages.html", **context)


@module_blueprint.route("dashboard/s/<slug>", methods=["GET"])
def view_page_dashboard(slug):
    context = {}
    page = Page.query.filter(Page.slug == slug).first()
    form = PageForm(obj=page)

    lang_arg = request.args.get("lang", get_current_lang())

    form.content = page.get_content(lang=lang_arg)
    form.lang.data = lang_arg

    context.update({"page": page, "form": form})
    return render_template("page/view_page_dashboard.html", **context)


@module_blueprint.route("/s/<slug>", methods=["GET"])
def view_page(slug):
    context = {}
    page = Page.query.filter(Page.slug == slug).first()
    context.update({"page": page})
    return render_template("page/view_page.html", **context)


@module_blueprint.route(mhelp.info["dashboard"])
@login_required
def dashboard():
    context = {}
    form = PageForm()

    context.update({"form": form, "module_name": module_name})
    context.update(module_settings)
    return render_template("page/dashboard.html", **context)


@module_blueprint.route("/check_pagecontent", methods=["GET", "POST"])
@login_required
def check_pagecontent():
    if request.method == "POST":
        form = PageForm()
        if not form.validate_on_submit():
            flash_errors(form)
            return redirect(url_for(f"{module_name}.dashboard"))
        toaddpage = Page(
            slug=form.slug.data,
            title=form.title.data,
        )
        db.session.add(toaddpage)
        db.session.flush()
        toaddpage.insert_lang(form.lang.data, form.content.data)
        toaddpage.save()
        return redirect(url_for(f"{module_name}.dashboard"))


@module_blueprint.route("/edit_pagecontent", methods=["GET", "POST"])
@login_required
def edit_pagecontent():
    if request.method == "POST":
        form = PageForm()
        if not form.validate_on_submit():
            flash_errors(form)
            return redirect(url_for(f"{module_name}.dashboard"))

        editpage = db.session.query(Page).get(request.form["page_id"])
        editpage.slug = form.slug.data
        editpage.title = form.title.data
        editpage.content = form.content.data

        editpage.set_lang(form.lang.data, form.content.data)
        db.session.commit()
        return redirect(
            url_for(
                f"{module_name}.view_page_dashboard",
                slug=form.slug.data,
                lang=form.lang.data,
            )
        )



# """
# This file (test_email.py) contains the unit tests for
# functions defined in email.py which is used for sending
# user email confirmation
# """
# import pytest
# from flask_mailman import EmailMessage
# from modules.box__default.auth.email import _send_email_helper
# from modules.box__default.auth.email import send_async_email
# from flask_login import UserMixin
# from init import db
# from shopyo.api.models import PkModel
# class ExampleUserModel(PkModel, UserMixin):
#     """Example user model for testing purposes"""
#     __tablename__ = "testusers_email"
#     password = db.Column(db.String(100), unique=True, nullable=False)
#     email = db.Column(db.String(120), unique=True, nullable=False)
# @pytest.mark.parametrize(
#     "email_config",
#     [("MAIL_DEFAULT_SENDER", "remove"), ("MAIL_DEFAULT_SENDER", None)],
#     indirect=True,
# )
# def test_send_email_with_no_default_sender(capfd, email_config):
#     user = User.create(email="test@gmail.com", password="pass")
#     token = "sometoken"
#     template = "auth/emails/activate_user"
#     subject = "Please confirm your email"
#     context = {"token": token, "user": user}
#     send_async_email(user.email, subject, template, **context)
#     captured = capfd.readouterr()
#     assert "Shopyo Error: MAIL_DEFAULT_SENDER not configured" in captured.out
# @pytest.mark.parametrize(
#     "email_config",
#     [
#         ("MAIL_USERNAME", "remove"),
#         ("MAIL_PASSWORD", "remove"),
#         ("MAIL_USERNAME", None),
#         ("MAIL_PASSWORD", None),
#     ],
#     indirect=True,
# )
# def test_send_email_with_no_username_or_password_set(capfd, email_config):
#     user = User.create(email="test@gmail.com", password="pass")
#     token = "sometoken"
#     template = "auth/emails/activate_user"
#     subject = "Please confirm your email"
#     context = {"token": token, "user": user}
#     thread = send_async_email(user.email, subject, template, **context)
#     thread.join()
#     captured = capfd.readouterr()
#     assert (
#         "Shopyo Error: MAIL_USERNAME, and/or MAIL_PASSWORD not configured"
#         in captured.out
#     )
# def test_send_email_using_template_on_valid_credentials(capfd):
#     user = User.create(email="to@gmail.com", password="pass")
#     token = "sometoken"
#     template = "auth/emails/activate_user"
#     subject = "Please confirm your email"
#     from_email = "from@gmail.com"
#     context = {"token": token, "user": user}
#     thread = send_async_email(
#         user.email, subject, template, from_email=from_email, **context
#     )
#     thread.join()
#     captured = capfd.readouterr()
#     assert "Please confirm your email" in captured.out
#     assert "sometoken" in captured.out
#     assert "to@gmail.com" in captured.out
#     assert "from@gmail.com" in captured.out
#     assert "Welcome to Shopyo" in captured.out
#     assert "To confirm your account please click on" in captured.out
#     assert "The Shopyo Team" in captured.out
# def test_send_using_helper_function(test_client, flask_app, capfd):
#     msg = EmailMessage(
#         subject="subject of email",
#         body="body of email",
#         to=["to@test.com"],
#         from_email="from@test.com",
#     )
#     _send_email_helper(flask_app, msg)
#     captured = capfd.readouterr()
#     assert "to@test.com" in captured.out
#     assert "from@test.com" in captured.out
#     assert "subject of email" in captured.out
#     assert "body of email" in captured.out



from urllib.parse import urljoin
from urllib.parse import urlparse

from flask import request


# from https://security.openstack.org/guidelines/dg_avoid-unvalidated-redirects.html
def is_safe_redirect_url(target):
    """
    Corresponds to Djangos is_safe_url


    Args:
        target (String): url

    Returns
    -------
    bool
    """
    host_url = urlparse(request.host_url)
    redirect_url = urlparse(urljoin(request.host_url, target))
    return (
        redirect_url.scheme in ("http", "https")
        and host_url.netloc == redirect_url.netloc
    )


def get_safe_redirect(url):
    """
    Returns url for root path if url not safe


    Args:
        url (String): url

    Returns
    -------
    url or root page
    """

    if url and is_safe_redirect_url(url):
        return url

    url = request.referrer
    if url and is_safe_redirect_url(url):
        return url

    return "/"



import json
import os

from flask import Blueprint
from flask import current_app
from flask import send_from_directory

# from flask import render_template
# from flask import redirect

# from flask import url_for

# from flask_login import login_required
# from PIL import Image as PILimage

# from shopyo.api.enhance import get_setting
# from shopyo.api.file import delete_file

# from modules.box__ecommerce.product.models import Product
# from modules.resource.models import Image

# from modules.resource.models import Resource

# from flask import flash
# from flask import request#
# from shopyo.api.html import notify_success
# from shopyo.api.forms import flash_errors


dirpath = os.path.dirname(os.path.abspath(__file__))
module_info = {}

with open(dirpath + "/info.json") as f:
    module_info = json.load(f)

globals()["{}_blueprint".format(module_info["module_name"])] = Blueprint(
    "{}".format(module_info["module_name"]),
    __name__,
    template_folder="templates",
    url_prefix=module_info["url_prefix"],
)


module_blueprint = globals()["{}_blueprint".format(module_info["module_name"])]


@module_blueprint.route("/")
def index():
    return module_info["display_string"]


@module_blueprint.route("/theme/front/<active_theme>/styles.css", methods=["GET"])
def active_front_theme_css(active_theme):
    theme_dir = os.path.join(
        current_app.config["BASE_DIR"],
        "static",
        "themes",
        "front",
        active_theme,
    )
    # return theme_dir
    return send_from_directory(theme_dir, "styles.css")


@module_blueprint.route("/theme/back/<active_theme>/styles.css", methods=["GET"])
def active_back_theme_css(active_theme):
    theme_dir = os.path.join(
        current_app.config["BASE_DIR"],
        "static",
        "themes",
        "back",
        active_theme,
    )
    # return theme_dir
    return send_from_directory(theme_dir, "styles.css")


# # Handles javascript image uploads from tinyMCE
# @module_blueprint.route("/upload/tinymce/image", methods=["POST"])
# @login_required
# def upload_tinymce_image():
#     # Kevin Foong
#     file = request.files.get("file")
#     if file:
#         filename = file.filename.lower()
#         fn, ext = filename.split(".")
#         # truncate filename (excluding extension) to 30 characters
#         fn = fn[:30]
#         filename = fn + "." + ext
#         if ext in ["jpg", "gif", "png", "jpeg"]:
#             try:
#                 # everything looks good, save file
#                 img_fullpath = os.path.join(
#                     current_app.config["UPLOADED_PATH_IMAGE"], filename
#                 )
#                 file.save(img_fullpath)
#                 # get the file size to save to db
#                 file_size = os.stat(img_fullpath).st_size
#                 size = 160, 160
#                 # read image into pillow
#                 im = PILimage.open(img_fullpath)
#                 # get image dimension to save to db
#                 file_width, file_height = im.size
#                 # convert to thumbnail
#                 im.thumbnail(size)
#                 thumbnail = fn + "-thumb.jpg"
#                 tmb_fullpath = os.path.join(
#                     current_app.config["UPLOADED_PATH_THUMB"], thumbnail
#                 )
#                 # PNG is index while JPG needs RGB
#                 if not im.mode == "RGB":
#                     im = im.convert("RGB")
#                 # save thumbnail
#                 im.save(tmb_fullpath, "JPEG")

#                 # save to db
#                 img = Image(
#                     filename=filename,
#                     thumbnail=thumbnail,
#                     file_size=file_size,
#                     file_width=file_width,
#                     file_height=file_height,
#                 )
#                 db.session.add(img)
#                 db.session.commit()
#             except IOError:
#                 output = make_response(404)
#                 output.headers["Error"] = (
#                     "Cannot create thumbnail for " + filename
#                 )
#                 return output
#             return jsonify({"location": filename})

#     # fail, image did not upload
#     output = make_response(404)
#     output.headers["Error"] = "Filename needs to be JPG, JPEG, GIF or PNG"
#     return output



import json
import os

from flask import Blueprint
from flask import current_app
from flask import redirect
from flask import render_template
from flask import url_for
from flask_login import login_required
from modules.box__default.settings.helpers import get_setting
from modules.box__default.settings.helpers import set_setting

from shopyo.api.file import get_folders

# from flask import flash
# from flask import request
# from shopyo.api.html import notify_success
# from init import db

# from modules.box__default.settings.models import Settings

# from shopyo.api.forms import flash_errors


dirpath = os.path.dirname(os.path.abspath(__file__))
module_info = {}

with open(dirpath + "/info.json") as f:
    module_info = json.load(f)

globals()["{}_blueprint".format(module_info["module_name"])] = Blueprint(
    "{}".format(module_info["module_name"]),
    __name__,
    template_folder="templates",
    url_prefix=module_info["url_prefix"],
)

module_settings = {"module_info": module_info}

module_blueprint = globals()["{}_blueprint".format(module_info["module_name"])]


@module_blueprint.route("/")
@login_required
def index():
    context = {}

    front_themes_path = os.path.join(
        current_app.config["BASE_DIR"], "static", "themes", "front"
    )
    all_front_info = {}
    front_theme_folders = get_folders(front_themes_path)
    for folder in front_theme_folders:
        theme_path = os.path.join(front_themes_path, folder)
        info_path = os.path.join(theme_path, "info.json")
        with open(info_path) as f:
            all_front_info[folder] = json.load(f)

    back_themes_path = os.path.join(
        current_app.config["BASE_DIR"], "static", "themes", "back"
    )
    all_back_info = {}
    back_theme_folders = get_folders(back_themes_path)
    for folder in back_theme_folders:
        theme_path = os.path.join(back_themes_path, folder)
        info_path = os.path.join(theme_path, "info.json")
        with open(info_path) as f:
            all_back_info[folder] = json.load(f)

    active_front_theme = get_setting("ACTIVE_FRONT_THEME")
    active_back_theme = get_setting("ACTIVE_BACK_THEME")

    context.update(
        {
            "all_front_info": all_front_info,
            "all_back_info": all_back_info,
            "active_front_theme": active_front_theme,
            "active_back_theme": active_back_theme,
        }
    )
    context.update(module_settings)

    return render_template(
        "{}/index.html".format(module_info["module_name"]), **context
    )


@module_blueprint.route("/activate/front/<theme_name>")
@login_required
def activate_front_theme(theme_name):
    set_setting("ACTIVE_FRONT_THEME", theme_name)

    # with app.app_context():

    # current_app.jinja_loader,
    # print(current_app.jinja_loader.list_templates())
    return redirect(url_for("{}.index".format(module_info["module_name"])))


@module_blueprint.route("/activate/back/<theme_name>")
@login_required
def activate_back_theme(theme_name):
    set_setting("ACTIVE_BACK_THEME", theme_name)

    # with app.app_context():

    # current_app.jinja_loader,
    # print(current_app.jinja_loader.list_templates())
    return redirect(url_for("{}.index".format(module_info["module_name"])))



"""
DB-related helper utilities. Taken from database.py
file at https://github.com/cookiecutter-flask/cookiecutter-flask
"""
import os
import sys

sys.path.append(os.getcwd())
from init import db


class CRUDMixin:
    """
    Mixin that adds convenience methods for
    CRUD (create, read, update, delete) operations.
    """

    @classmethod
    def create(cls, **kwargs):
        """Create a new record and save it in the database.

        Returns:
            DB Class Object: returns the created record
        """
        instance = cls(**kwargs)
        return instance.save()

    def update(self, commit=True, **kwargs):
        """Update specific fields of a record

        Args:
            commit (bool, optional): flag whether to commit. Defaults to True.

        Returns:
            Db Class object: returns the updated record if committed,
            None otherwise
        """
        for attr, value in kwargs.items():
            setattr(self, attr, value)
        if commit:
            self.save()
            return self
        return None

    def save(self, commit=True):
        """Save the record.

        Args:
            commit (bool, optional): flag whether to commit. Defaults to True.

        Returns:
            Db Class object: returns the record saved to db session
        """
        db.session.add(self)
        if commit:
            db.session.commit()
        return self

    def delete(self, commit=True):
        """Remove the record from the database.

        Args:
            commit (bool, optional): flag whether to commit. Defaults to True.

        Returns:
            Db Class object: returns the updated record if committed,
            None otherwise
        """
        db.session.delete(self)
        if commit:
            db.session.commit()
            return self
        return None


class YoModel(CRUDMixin, db.Model):
    """Base model class that includes CRUD convenience methods."""

    __abstract__ = True


class PkModel(YoModel):
    """
    Base model class that includes CRUD convenience methods,
    plus adds a 'primary key' column named 'id'.
    """

    __abstract__ = True
    id = db.Column(db.Integer, primary_key=True)

    @classmethod
    def get_by_id(cls, record_id):
        """Get record by ID.

        Args:
            record_id (int): ID of record to get

        Returns:
            DB Class object: object identified by record_id if any,
            None otherwise
        """
        if any(
            (
                isinstance(record_id, (str, bytes)) and record_id.isdigit(),
                isinstance(record_id, (int, float)),
            )
        ):
            return cls.query.get(int(record_id))
        return None



import os
import shutil

import pytest
from app import create_app

from shopyo.api.file import tryrmtree

# from shopyo.app import app as _app


@pytest.fixture(scope="module")
def temp_app():
    return create_app("testing")


@pytest.fixture
def app(tmpdir, app_type, temp_app):
    src = os.path.join(temp_app.instance_path, "config.py")
    dest = tmpdir.join("temp_config.py")
    dest.write("")
    shutil.copy(src, dest)
    tryrmtree(temp_app.instance_path)
    dev_app = create_app(app_type)
    yield dev_app
    shutil.copy(dest, src)



import datetime

from init import db


class Image(db.Model):
    __tablename__ = "images"
    id = db.Column(db.Integer, primary_key=True)
    filename = db.Column(db.String(50), nullable=False)
    thumbnail = db.Column(db.String(50), nullable=False)
    file_size = db.Column(db.Integer, nullable=False)
    file_width = db.Column(db.Integer, nullable=False)
    file_height = db.Column(db.Integer, nullable=False)
    create_date = db.Column(
        db.DateTime, default=datetime.datetime.now(), nullable=False
    )

    # def set_hash(self, password):
    #     self.password = generate_password_hash(password, method="sha256")

    # def check_hash(self, password):
    #     return check_password_hash(self.password, password)

    def insert(self):
        db.session.add(self)
        db.session.commit()

    def update(self):
        db.session.commit()

    def delete(self):
        db.session.delete(self)
        db.session.commit()

    # def getImage(self, image_id):
    #     return Images.query.filter_by(id=image_id).first()


class Resource(db.Model):
    __tablename__ = "resources"
    id = db.Column(db.Integer, primary_key=True)
    filename = db.Column(db.String(50), nullable=False)
    type = db.Column(db.String(50), nullable=False)
    category = db.Column(db.String(50), nullable=False)

    created_date = db.Column(
        db.DateTime, default=datetime.datetime.now(), nullable=False
    )

    # def set_hash(self, password):
    #     self.password = generate_password_hash(password, method="sha256")

    # def check_hash(self, password):
    #     return check_password_hash(self.password, password)

    def insert(self):
        db.session.add(self)
        db.session.commit()

    def update(self):
        db.session.commit()

    def delete(self):
        db.session.delete(self)
        db.session.commit()



import json
import os

from flask import url_for
from init import themes_path
from modules.box__default.settings.helpers import get_setting


def get_front_theme_dir():
    theme_dir = os.path.join(themes_path, "front", get_setting("ACTIVE_FRONT_THEME"))
    return theme_dir


def get_front_theme_info_data():
    info_path = os.path.join(get_front_theme_dir(), "info.json")
    with open(info_path) as f:
        info_data = json.load(f)
    return info_data


def get_active_front_theme():
    return get_setting("ACTIVE_FRONT_THEME")


def get_active_front_theme_version():
    return get_front_theme_info_data()["version"]


def get_active_front_theme_styles_url():
    return url_for(
        "resource.active_front_theme_css",
        active_theme=get_active_front_theme(),
        v=get_active_front_theme_version(),
    )


def get_back_theme_dir():
    theme_dir = os.path.join(themes_path, "back", get_setting("ACTIVE_BACK_THEME"))
    return theme_dir


def get_back_theme_info_data():
    info_path = os.path.join(get_back_theme_dir(), "info.json")
    with open(info_path) as f:
        info_data = json.load(f)
    return info_data


def get_active_back_theme():
    return get_setting("ACTIVE_BACK_THEME")


def get_active_back_theme_version():
    return get_back_theme_info_data()["version"]


def get_active_back_theme_styles_url():
    return url_for(
        "resource.active_back_theme_css",
        active_theme=get_active_back_theme(),
        v=get_active_back_theme_version(),
    )


available_everywhere = {
    "get_active_front_theme": get_active_front_theme,
    "get_active_front_theme_version": get_active_front_theme_version,
    "get_active_front_theme_styles_url": get_active_front_theme_styles_url,
    "get_active_back_theme": get_active_back_theme,
    "get_active_back_theme_version": get_active_back_theme_version,
    "get_active_back_theme_styles_url": get_active_back_theme_styles_url,
}



import textwrap


def get_module_view_content():
    content = textwrap.dedent(
        """\
        from shopyo.api.module import ModuleHelp
        # from flask import render_template
        # from flask import url_for
        # from flask import redirect
        # from flask import flash
        # from flask import request

        # from shopyo.api.html import notify_success
        # from shopyo.api.forms import flash_errors

        mhelp = ModuleHelp(__file__, __name__)
        globals()[mhelp.blueprint_str] = mhelp.blueprint
        module_blueprint = globals()[mhelp.blueprint_str]


        @module_blueprint.route("/")
        def index():
            return mhelp.info['display_string']

        # If "dashboard": "/dashboard" is set in info.json
        #
        # @module_blueprint.route("/dashboard", methods=["GET"])
        # def dashboard():

        #     context = mhelp.context()

        #     context.update({

        #         })
        #     return mhelp.render('dashboard.html', **context)
        """
    )

    return content


def get_dashboard_html_content():
    content = textwrap.dedent(
        """\
        {% extends "base/module_base.html" %}
        {% set active_page = info['display_string']+' dashboard' %}
        {% block pagehead %}
        <title></title>
        <style>
        </style>
        {% endblock %}
        {% block sidebar %}
        {% include info['module_name']+'/blocks/sidebar.html' %}
        {% endblock %}
        {% block content %}
        <br>

        <div class="card">
            <div class="card-body">

            </div>
        </div>
        {% endblock %}
        """
    )

    return content


def get_global_py_content():
    content = textwrap.dedent(
        """\

        # global templates variables in here
        # available_everywhere = {
        #
        # }

        # global configs in here, defined by profile
        # configs = {
        #     "development": {
        #         "CONFIG_VAR": "DEVVALUE"
        #     },
        #     "production": {
        #         "CONFIG_VAR": "PRODVALUE"
        #     },
        #     "testing": {
        #         "CONFIG_VAR": "TESTVALUE"
        #     }
        # }
        """
    )

    return content


def get_cli_content(projname):
    content = textwrap.dedent(
        f'''\
        """
        file: cli.py
        description: Add your custom cli commands here
        documentation: https://click.palletsprojects.com/en/7.x/

        You will need to run ``python -m pip install -e .`` to load the setup.py
        which contains the entry point to this file before being able to run your
        custom commands

        Usage ``{projname} [OPTIONS] COMMAND [ARGS]...``

        Example command 'welcome' has been added.
        - To get your project version, run ``{projname} --version``
        - Run the sample command as ``{projname} welcome [OPTIONS] NAME``
        """

        from {projname} import __version__
        import click


        @click.group()
        @click.version_option(__version__)
        @click.pass_context
        def cli(ctx):
            """CLI entry point"""
            pass


        @cli.command("welcome")
        @click.argument("name")
        @click.option('--verbose', "-v", is_flag=True, default=False)
        def welcome(name, verbose):
            """Sample command to welcome users.

            NAME will be printed along with the welcome message
            """
            click.secho(f"Hi {{name}}. Welcome to {projname}", fg="cyan")

            if verbose:
                click.echo("See you soon")
        '''
    )

    return content


def get_init_content():
    content = textwrap.dedent(
        """\
        version_info = (1, 0, 0)
        __version__ = ".".join([str(v) for v in version_info])
        """
    )

    return content


def get_setup_py_content(projname):
    content = textwrap.dedent(
        f'''\
        """
        A setuptools based setup module.
        See:
        https://packaging.python.org/guides/distributing-packages-using-setuptools/
        https://github.com/pypa/sampleproject

        python setup.py publish to publish

        """

        # Always prefer setuptools over distutils
        from setuptools import setup

        # from setuptools import find_packages

        import os
        import sys
        from {projname} import __version__  # thanks gunicorn

        here = os.path.abspath(os.path.dirname(__file__))

        if sys.argv[-1] == "publish":  # requests
            os.system("python setup.py sdist")  # bdist_wheel
            os.system("twine upload dist/* --skip-existing")
            sys.exit()

        # Get the long description from the README file
        with open(os.path.join(here, "README.md"), encoding="utf-8") as f:
            long_description = f.read()
        setup(
            name="{projname}",  # Required
            version=__version__,  # Required
            description="",  # Optional
            long_description=long_description,  # Optional
            long_description_content_type="text/markdown",  # Optional (see note above)
            url="",  # Optional
            author="",  # Optional
            author_email="",  # Optional
            # Classifiers help users find your project by categorizing it.
            #
            # For a list of valid classifiers, see https://pypi.org/classifiers/
            classifiers=[  # Optional
                # How mature is this project? Common values are
                #   3 - Alpha
                #   4 - Beta
                #   5 - Production/Stable
                "Development Status :: 4 - Beta",
                # Indicate who your project is intended for
                "Intended Audience :: Developers",
                # 'Topic :: Weather',
                # Pick your license as you wish
                "License :: OSI Approved :: MIT License",
                # Specify the Python versions you support here. In particular, ensure
                # that you indicate whether you support Python 2, Python 3 or both.
                # These classifiers are *not* checked by 'pip install'. See instead
                # 'python_requires' below.
                "Programming Language :: Python :: 3.6",
                "Programming Language :: Python :: 3.7",
                "Programming Language :: Python :: 3.8",
                "Programming Language :: Python :: 3.9",
            ],
            keywords="",  # Optional
            # You can just specify package directories manually here if your project is
            # simple. Or you can use find_packages().
            #
            # Alternatively, if you just want to distribute a single Python file, use
            # the `py_modules` argument instead as follows, which will expect a file
            # called `my_module.py` to exist:
            #
            #   py_modules=["my_module"],
            #
            # packages=find_packages(exclude=['contrib', 'docs', 'tests']),  # Required
            packages=["{projname}"],
            include_package_data=True,
            python_requires=">=3.6",
            install_requires=open(
                os.path.join(here, "requirements.txt"), encoding="utf-8"
            )
            .read()
            .split("\\n"),  # Optional
            project_urls={{  # Optional
                "Bug Reports": "",
                "Source": "",
            }},
            entry_points={{
                "console_scripts": [
                    "{projname}={projname}.cli:cli"
                ]
            }},
        )
        '''
    )

    return content


def get_manifest_ini_content(projname):
    content = textwrap.dedent(
        """\
        include requirements.txt
        include dev_requirements.txt
        exclude config.json
        recursive-include {projname} *
        recursive-exclude {projname}/instance *
        recursive-exclude {projname}/static/modules *
        recursive-exclude {projname}/.tox *
        recursive-exclude __pycache__ *
        """
    )

    return content


def get_pytest_ini_content():
    content = textwrap.dedent(
        """\
        [pytest]
        env_files =
            .test.prod.env
        """
    )

    return content


def get_tox_ini_content(projname):
    content = textwrap.dedent(
        f"""\
        [tox]
        envlist =
            py39
            py38
            py37
            py36
        skip_missing_interpreters=true

        [testenv]
        changedir = {projname}
        deps =
            -rrequirements.txt
            -rdev_requirements.txt
        commands = python -m pytest {{posargs}}
        """
    )

    return content


def get_dev_req_content():
    content = textwrap.dedent(
        """\
        flake8==3.8.4
        black==20.8b1
        isort==5.6.4
        Sphinx==3.2.1
        pytest==6.1.1
        pytest-order==0.9.5
        tox==3.21.0
        pytest-cov==2.11.1
        codecov==2.1.11
        factory-boy==3.2.0
        freezegun==1.1.0
        pytest-dotenv
        """
    )

    return content


def get_gitignore_content():
    content = textwrap.dedent(
        """\
        # Byte-compiled / optimized / DLL files
        __pycache__/
        *.py[cod]
        *$py.class

        # C extensions
        *.so

        # Distribution / packaging
        .Python
        build/
        develop-eggs/
        dist/
        downloads/
        eggs/
        .eggs/
        lib/
        lib64/
        parts/
        sdist/
        var/
        wheels/
        *.egg-info/
        .installed.cfg
        *.egg
        MANIFEST

        # PyInstaller
        #  Usually these files are written by a python script from a template
        #  before PyInstaller builds the exe, so as to inject date/other infos into it.
        *.manifest
        *.spec

        # Installer logs
        pip-log.txt
        pip-delete-this-directory.txt

        # Unit test / coverage reports
        htmlcov/
        .tox/
        .coverage
        .coverage.*
        .cache
        nosetests.xml
        coverage.xml
        *.cover
        .hypothesis/
        .pytest_cache/

        # Translations
        *.mo
        *.pot

        # Django stuff:
        *.log
        local_settings.py
        db.sqlite3

        # Flask stuff:
        instance/
        .webassets-cache

        # Scrapy stuff:
        .scrapy

        # Sphinx documentation
        _build/

        # PyBuilder
        target/

        # Jupyter Notebook
        .ipynb_checkpoints

        # pyenv
        .python-version

        # celery beat schedule file
        celerybeat-schedule

        # SageMath parsed files
        *.sage.py

        # Environments
        .env
        .venv
        env/
        venv/
        ENV/
        env.bak/
        venv.bak/

        # Spyder project settings
        .spyderproject
        .spyproject

        # Rope project settings
        .ropeproject

        # mkdocs documentation
        /site

        # mypy
        .mypy_cache/

        # shopyo
        test.db
        testing.db
        shopyo.db

        # pycharm
        .idea/

        # win
        *.exe
        *.cs
        *.bat
        *.vbs

        # vscode
        .vscode
        workspace.code-workspace

        # modules in static since present in modules
        shopyo/static/modules/

        # ignore secrets
        config.json
        """
    )

    return content


def get_index_rst_content(projname):
    content = textwrap.dedent(
        f"""\
        Welcome to {projname} docs!
        ============================

        Write project description here

        .. toctree::
           :maxdepth: 9

           Docs <docs>

        Indices and tables
        ==================

        * :ref:`genindex`
        * :ref:`modindex`
        * :ref:`search`
        """
    )

    return content


def get_docs_rst_content(projname):
    content = textwrap.dedent(
        """\
        .. :tocdepth:: 5

        Documentation
        =============

        Sphinx is included in dev_requirements.txt

        .. code:: bash

            cd docs
            sphinx-build . _build

        to generate html pages in docs
        """
    )

    return content


def get_sphinx_conf_py(projname):
    content = textwrap.dedent(
        f"""\
        # Configuration file for the Sphinx documentation builder.
        #
        # This file only contains a selection of the most common options. For a full
        # list see the documentation:
        # https://www.sphinx-doc.org/en/master/usage/configuration.html

        # -- Path setup --------------------------------------------------------------
        #
        # If extensions (or modules to document with autodoc) are in another directory,
        # add these directories to sys.path here. If the directory is relative to the
        # documentation root, use os.path.abspath to make it absolute, like shown here.
        #
        # import os
        # import sys
        # sys.path.insert(0, os.path.abspath('.'))
        #
        # -- Project information -----------------------------------------------------

        project = "{projname}"
        author = ""

        # -- General configuration ---------------------------------------------------

        # Add any Sphinx extension module names here, as strings. They can be
        # extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
        # ones.
        extensions = [
            "sphinx.ext.autodoc",
            "sphinx.ext.viewcode",
            "sphinx.ext.autosectionlabel",
            "sphinx.ext.napoleon",
        ]

        # Add any paths that contain templates here, relative to this directory.
        templates_path = ["_templates"]

        # List of patterns, relative to source directory, that match files and
        # directories to ignore when looking for source files.
        # This pattern also affects html_static_path and html_extra_path.
        exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]


        # -- Options for HTML output -------------------------------------------------
        html_context = {{
            "project_links": [
                "Source Code",
                "https://github.com/<name>/<project>",
                "Issue Tracker",
                "https://github.com/<name>/<project>/issues",
            ]
        }}
        html_logo = "shopyo.ico"

        html_sidebars = {{
            "**": ["about.html", "relations.html", "navigation.html", "searchbox.html"]
        }}

        # The theme to use for HTML and HTML Help pages.  See the documentation for
        # a list of builtin themes.
        #
        html_theme = "alabaster"
        html_theme_options = {{
            "github_repo": "<name>/<project>",
            "fixed_sidebar": "true",
        }}

        # Add any paths that contain custom static files (such as style sheets) here,
        # relative to this directory. They are copied after the builtin static files,
        # so a file named "default.css" will overwrite the builtin "default.css".
        html_static_path = ["_static"]
        """
    )

    return content


def get_sphinx_makefile():
    content = textwrap.dedent(
        """\
        # Minimal makefile for Sphinx documentation
        #

        # You can set these variables from the command line, and also
        # from the environment for the first two.
        SPHINXOPTS    ?=
        SPHINXBUILD   ?= sphinx-build
        SOURCEDIR     = .
        BUILDDIR      = _build

        # Put it first so that "make" without argument is like "make help".
        help:
            @$(SPHINXBUILD) -M help "$(SOURCEDIR)" "$(BUILDDIR)" $(SPHINXOPTS) $(O)

        .PHONY: help Makefile

        # Catch-all target: route all unknown targets to Sphinx using the new
        # "make mode" option.  $(O) is meant as a shortcut for $(SPHINXOPTS).
        %: Makefile
            @$(SPHINXBUILD) -M $@ "$(SOURCEDIR)" "$(BUILDDIR)" $(SPHINXOPTS) $(O)

        """
    )

    return content



import datetime

from flask import current_app
from flask import flash
from flask import redirect
from flask import render_template
from flask import request
from flask import url_for
from flask_login import current_user
from flask_login import login_required
from flask_login import login_user
from flask_login import logout_user
from sqlalchemy import func

from .forms import LoginForm
from .forms import RegistrationForm
from .models import User
from shopyo.api.email import send_async_email
from shopyo.api.html import notify_danger
from shopyo.api.html import notify_success
from shopyo.api.html import notify_warning
from shopyo.api.module import ModuleHelp
from shopyo.api.security import get_safe_redirect


mhelp = ModuleHelp(__file__, __name__)
globals()[mhelp.blueprint_str] = mhelp.blueprint
module_blueprint = globals()[mhelp.blueprint_str]


@module_blueprint.route("/register", methods=["GET", "POST"])
def register():
    context = {}
    reg_form = RegistrationForm()

    if reg_form.validate_on_submit():
        email = reg_form.email.data
        password = reg_form.password.data
        user = User.create(email=email, password=password)
        login_user(user)

        is_disabled = False

        if "EMAIL_CONFIRMATION_DISABLED" in current_app.config:
            is_disabled = current_app.config["EMAIL_CONFIRMATION_DISABLED"]

        if is_disabled is True:
            user.is_email_confirmed = True
            user.email_confirm_date = datetime.datetime.now()
            user.update()
        else:
            token = user.generate_confirmation_token()
            template = "auth/emails/activate_user"
            subject = "Please confirm your email"
            context.update({"token": token, "user": user})
            send_async_email(email, subject, template, **context)
            flash(notify_success("A confirmation email has been sent via email."))

        return redirect(url_for("dashboard.index"))

    context["form"] = reg_form
    return render_template("auth/register.html", **context)


@module_blueprint.route("/confirm/<token>")
@login_required
def confirm(token):
    if current_user.is_email_confirmed:
        flash(notify_warning("Account already confirmed."))
        return redirect(url_for("dashboard.index"))

    if current_user.confirm_token(token):
        flash(notify_success("You have confirmed your account. Thanks!"))
        return redirect(url_for("dashboard.index"))

    flash(notify_warning("The confirmation link is invalid/expired."))
    return redirect(url_for("auth.unconfirmed"))


@module_blueprint.route("/resend")
@login_required
def resend():
    if current_user.is_email_confirmed:
        return redirect(url_for("dashboard.index"))

    token = current_user.generate_confirmation_token()
    template = "auth/emails/activate_user"
    subject = "Please confirm your email"
    context = {"token": token, "user": current_user}
    send_async_email(current_user.email, subject, template, **context)
    flash(notify_success("A new confirmation email has been sent."))
    return redirect(url_for("auth.unconfirmed"))


@module_blueprint.route("/unconfirmed")
@login_required
def unconfirmed():
    if current_user.is_email_confirmed:
        return redirect(url_for("dashboard.index"))
    flash(notify_warning("Please confirm your account!"))
    return render_template("auth/unconfirmed.html")


@module_blueprint.route("/login", methods=["GET", "POST"])
def login():
    context = {}
    login_form = LoginForm()
    context["form"] = login_form
    if login_form.validate_on_submit():
        email = login_form.email.data
        password = login_form.password.data
        user = User.query.filter(func.lower(User.email) == func.lower(email)).first()
        if user is None or not user.check_password(password):
            flash(notify_danger("please check your user id and password"))
            return redirect(url_for("auth.login"))
        login_user(user)
        if "next" not in request.form:
            next_url = url_for("dashboard.index")

        else:
            if request.form["next"] == "":
                next_url = url_for("dashboard.index")
            else:
                next_url = get_safe_redirect(request.form["next"])
        return redirect(next_url)
    return render_template("auth/login.html", **context)


@module_blueprint.route("/logout", methods=["GET"])
@login_required
def logout():
    logout_user()
    flash(notify_success("Successfully logged out"))

    if "next" not in request.args:
        next_url = url_for("dashboard.index")
    else:
        if request.args.get("next") == "":
            next_url = url_for("dashboard.index")
        else:
            next_url = get_safe_redirect(request.args.get("next"))
    return redirect(next_url)



"""
This file (test_admin.py) contains the functional tests for
the `admin` blueprint.

These tests use GETs and POSTs to different endpoints to check
for the proper behavior of the `admin` blueprint.
"""
import json
import os

import pytest
from flask import request
from flask import url_for
from modules.box__default.auth.models import Role
from modules.box__default.auth.models import role_user_bridge
from modules.box__default.auth.models import User


dirpath = os.path.dirname(os.path.abspath(__file__))
module_path = os.path.dirname(dirpath)

module_info = None


# class TestAdminInvalidAccess:
#     """
#     Test all admin routes for correct user authentication
#     """

#     routes_get = [
#         "/",
#         "/add",
#         "/delete/<id>",
#         "/edit/<id>",
#         "/roles",
#         "/roles/<role_id>/delete",
#     ]

#     routes_post = ["/update", "/roles/update", "/roles/add", "/add"]

#     @pytest.mark.parametrize("route", routes_get)
#     def test_redirect_if_not_logged_in_get(self, test_client, route, auth):
#         auth.logout()
#         with open(os.path.join(module_path, "info.json")) as f:
#             module_info = json.load(f)
#         response = test_client.get(
#             f"{module_info['url_prefix']}{route}", follow_redirects=True
#         )

#         assert response.status_code == 200
#         assert request.path == url_for("auth.login")

#     @pytest.mark.parametrize("route", routes_post)
#     def test_redirect_if_not_logged_in_post(self, test_client, route, auth):
#         auth.logout()
#         with open(os.path.join(module_path, "info.json")) as f:
#             module_info = json.load(f)
#         response = test_client.post(
#             f"{module_info['url_prefix']}{route}", follow_redirects=True
#         )

#         assert response.status_code == 200
#         assert request.path == url_for("auth.login")

#     @pytest.mark.usefixtures("login_non_admin_user")
#     @pytest.mark.parametrize("route", routes_get)
#     def test_no_admin_access_if_not_admin_get(self, test_client, route):
#         with open(os.path.join(module_path, "info.json")) as f:
#             module_info = json.load(f)
#         response = test_client.get(
#             f"{module_info['url_prefix']}{route}", follow_redirects=True
#         )

#         assert response.status_code == 200
#         assert request.path == url_for("dashboard.index")
#         assert b"You need to be an admin to view this page" in response.data

#     @pytest.mark.usefixtures("login_non_admin_user")
#     @pytest.mark.parametrize("route", routes_post)
#     def test_no_admin_access_if_not_admin_post(self, test_client, route):
#         with open(os.path.join(module_path, "info.json")) as f:
#             module_info = json.load(f)
#         response = test_client.post(
#             f"{module_info['url_prefix']}{route}", follow_redirects=True
#         )

#         assert response.status_code == 200
#         assert request.path == url_for("dashboard.index")
#         assert b"You need to be an admin to view this page" in response.data


@pytest.mark.usefixtures("login_admin_user")
class TestAdminEndpoints:
    """
    Test all admin api post and get requests
    """

    # def test_admin_user_list_get(self, test_client):
    #     with open(os.path.join(module_path, "info.json")) as f:
    #         module_info = json.load(f)
    #     response = test_client.get(f"{module_info['url_prefix']}/")

    #     assert response.status_code == 200
    #     assert b"Admin" in response.data
    #     assert b"id" in response.data
    #     assert b"Email" in response.data
    #     assert b"Password" in response.data
    #     assert b"Roles" in response.data

    # def test_admin_add_get(self, test_client):
    #     with open(os.path.join(module_path, "info.json")) as f:
    #         module_info = json.load(f)
    #     response = test_client.get(f"{module_info['url_prefix']}/add")

    #     assert response.status_code == 200
    #     assert b"Email" in response.data
    #     assert b"Password" in response.data
    #     assert b"First Name" in response.data
    #     assert b"Last Name" in response.data
    #     assert b"Admin User" in response.data

    # def test_admin_add_unique_user_post(self, test_client):
    #     with open(os.path.join(module_path, "info.json")) as f:
    #         module_info = json.load(f)
    #     role1 = Role.create(name="test1-role")
    #     role2 = Role.create(name="test2-role")
    #     data = {
    #         "email": "test@gmail.com",
    #         "password": "pass",
    #         "first_name": "Test",
    #         "last_name": "User",
    #         "is_admin": "",
    #         f"role_{role1.id}": "",
    #         f"role_{role2.id}": "",
    #     }

    #     test_client.post(
    #         f"{module_info['url_prefix']}/add",
    #         data=data,
    #         follow_redirects=True,
    #     )
    #     test_user = User.query.filter(User.email == "test@gmail.com").scalar()

    #     assert test_user is not None
    #     assert test_user.first_name == "Test"
    #     assert test_user.last_name == "User"
    #     assert test_user.is_admin is False
    #     assert test_user.roles is not None
    #     assert len(test_user.roles) == 2
    #     assert role1.users[0].email == "test@gmail.com"
    #     assert role2.users[0].email == "test@gmail.com"

    # def test_admin_add_existing_user_post(self, test_client):
    #     with open(os.path.join(module_path, "info.json")) as f:
    #         module_info = json.load(f)
    #     User.create(email="test@gmail.com", password="pass")
    #     data = {
    #         "email": "test@gmail.com",
    #         "password": "pass",
    #         "first_name": "Test",
    #         "last_name": "User",
    #         "is_admin": "",
    #     }

    #     response = test_client.post(
    #         f"{module_info['url_prefix']}/add",
    #         data=data,
    #         follow_redirects=True,
    #     )
    #     test_users = User.query.filter(User.email == "test@gmail.com").count()

    #     assert response.status_code == 200
    #     assert b"User with same email already exists" in response.data
    #     assert test_users == 1

    # def test_admin_delete_existing_user_get(self, test_client):
    #     with open(os.path.join(module_path, "info.json")) as f:
    #         module_info = json.load(f)
    #     user = User(email="test@gmail.com", password="pass")
    #     role1 = Role(name="test1-role")
    #     role2 = Role(name="test2-role")
    #     user.roles = [role1, role2]
    #     user.save()

    #     response = test_client.get(
    #         f"{module_info['url_prefix']}/delete/{user.id}",
    #         follow_redirects=True,
    #     )
    #     test_user = User.query.filter(User.email == user.email).scalar()
    #     test_roles = Role.query.count()
    #     user_role = (
    #         User.query.join(role_user_bridge)
    #         .join(Role)
    #         .filter(User.id == user.id)
    #         .scalar()
    #     )

    #     assert response.status_code == 200
    #     assert test_user is None
    #     assert user_role is None
    #     assert test_roles == 2

    # def test_admin_delete_nonexisting_user_get(self, test_client):
    #     with open(os.path.join(module_path, "info.json")) as f:
    #         module_info = json.load(f)
    #     response = test_client.get(
    #         f"{module_info['url_prefix']}/delete/some_id",
    #         follow_redirects=True,
    #     )

    #     assert response.status_code == 200
    #     assert b"Unable to delete. Invalid user id" in response.data

    # def test_admin_edit_existing_user_get(self, test_client):
    #     with open(os.path.join(module_path, "info.json")) as f:
    #         module_info = json.load(f)
    #     user = User.create(email="test@gmail.com", password="pass")

    #     response = test_client.get(
    #         f"{module_info['url_prefix']}/edit/{user.id}",
    #     )

    #     assert response.status_code == 200
    #     assert b"test@gmail.com" in response.data
    #     assert b"Edit User" in response.data

    # def test_admin_edit_nonexisting_user_get(self, test_client):
    #     with open(os.path.join(module_path, "info.json")) as f:
    #         module_info = json.load(f)
    #     response = test_client.get(
    #         f"{module_info['url_prefix']}/edit/some-id", follow_redirects=True
    #     )

    #     assert response.status_code == 200
    #     assert b"Invalid user id" in response.data
    #     assert request.path == f"{module_info['url_prefix']}/"

    # def test_admin_update_user_adding_new_roles_to_user(self, test_client):
    #     with open(os.path.join(module_path, "info.json")) as f:
    #         module_info = json.load(f)
    #     user = User.create(email="foo@gmail.com", password="pass")
    #     role1 = Role.create(name="test1-role")
    #     role2 = Role.create(name="test2-role")
    #     data = {
    #         "id": str(user.id),
    #         "email": "bar@gmail.com",
    #         "password": "newpass",
    #         "first_name": "Test",
    #         "last_name": "User",
    #         "is_admin": "",
    #         f"role_{role1.id}": "",
    #         f"role_{role2.id}": "",
    #     }

    #     response = test_client.post(
    #         f"{module_info['url_prefix']}/update",
    #         data=data,
    #         follow_redirects=True,
    #     )

    #     assert response.status_code == 200
    #     assert user.email == "bar@gmail.com"
    #     assert user.check_password("newpass")
    #     assert user.first_name == "Test"
    #     assert user.last_name == "User"
    #     assert len(user.roles) == 2
    #     assert role1.users[0].email == "bar@gmail.com"
    #     assert role2.users[0].email == "bar@gmail.com"

    # def test_admin_update_user_remove_old_roles_from_user(self, test_client):
    #     with open(os.path.join(module_path, "info.json")) as f:
    #         module_info = json.load(f)
    #     user = User(email="foo@gmail.com", password="pass", is_admin=True)
    #     user.is_admin = True
    #     role1 = Role(name="test1-role")
    #     role2 = Role(name="test2-role")
    #     user.roles = [role1, role2]
    #     user.save()
    #     data = {
    #         "id": str(user.id),
    #         "email": "bar@gmail.com",
    #         "first_name": "Test",
    #         "last_name": "User",
    #         "password": "   ",
    #         "is_admin": None,
    #     }

    #     response = test_client.post(
    #         f"{module_info['url_prefix']}/update",
    #         data=data,
    #         follow_redirects=True,
    #     )

    #     assert response.status_code == 200
    #     assert user.email == "bar@gmail.com"
    #     assert user.check_password("pass")
    #     assert len(user.roles) == 0
    #     assert len(role1.users) == 0
    #     assert len(role2.users) == 0

    # def test_admin_roles_get(self, test_client):
    #     with open(os.path.join(module_path, "info.json")) as f:
    #         module_info = json.load(f)
    #     response = test_client.get(f"{module_info['url_prefix']}/roles")

    #     assert response.status_code == 200
    #     assert b"Roles" in response.data

    # def test_admin_roles_add_nonexisting_role_post(self, test_client):
    #     with open(os.path.join(module_path, "info.json")) as f:
    #         module_info = json.load(f)
    #     response = test_client.post(
    #         f"{module_info['url_prefix']}/roles/add",
    #         data=dict(name="new-role"),
    #         follow_redirects=True,
    #     )

    #     role = Role.query.filter(Role.name == "new-role").scalar()
    #     role_count = Role.query.count()

    #     assert response.status_code == 200
    #     assert role is not None
    #     assert role_count == 1

    # def test_admin_roles_add_existing_role_post(self, test_client):
    #     with open(os.path.join(module_path, "info.json")) as f:
    #         module_info = json.load(f)
    #     Role.create(name="new-role")

    #     response = test_client.post(
    #         f"{module_info['url_prefix']}/roles/add",
    #         data=dict(name="new-role"),
    #         follow_redirects=True,
    #     )
    #     role_count = Role.query.count()
    #     role = Role.query.filter(Role.name == "new-role").scalar()

    #     assert response.status_code == 200
    #     assert b"Role already exists" in response.data
    #     assert role is not None
    #     assert role_count == 1

    # def test_admin_roles_delete_nonexisting_role_get(self, test_client):
    #     with open(os.path.join(module_path, "info.json")) as f:
    #         module_info = json.load(f)
    #     response = test_client.get(
    #         f"{module_info['url_prefix']}/roles/some-id/delete",
    #         follow_redirects=True,
    #     )

    #     assert response.status_code == 200
    #     assert request.path == f"{module_info['url_prefix']}/roles"
    #     assert b"Unable to delete. Invalid role id" in response.data

    # def test_admin_roles_delete_existing_role_get(self, test_client):
    #     with open(os.path.join(module_path, "info.json")) as f:
    #         module_info = json.load(f)
    #     role1 = Role.create(name="new-role1")
    #     role2 = Role.create(name="new-role2")

    #     response = test_client.get(
    #         f"{module_info['url_prefix']}/roles/{role1.id}/delete",
    #         follow_redirects=True,
    #     )
    #     roles = Role.query.all()

    #     assert response.status_code == 200
    #     assert request.path == f"{module_info['url_prefix']}/roles"
    #     assert b"Role successfully deleted" in response.data
    #     assert roles is not None
    #     assert roles[0].name == role2.name
    #     assert len(roles) == 1

    # def test_admin_roles_update_nonexisting_role_post(self, test_client):
    #     with open(os.path.join(module_path, "info.json")) as f:
    #         module_info = json.load(f)
    #     response = test_client.post(
    #         f"{module_info['url_prefix']}/roles/update",
    #         data=dict(role_id="some-id"),
    #         follow_redirects=True,
    #     )
    #     roles = Role.query.count()

    #     assert response.status_code == 200
    #     assert request.path == f"{module_info['url_prefix']}/roles"
    #     assert b"Unable to update. Role does not exist" in response.data
    #     assert roles == 0

    # def test_admin_roles_update_existing_role_post(self, test_client):
    #     with open(os.path.join(module_path, "info.json")) as f:
    #         module_info = json.load(f)
    #     new_role = Role.create(name="new-role1")

    #     response = test_client.post(
    #         f"{module_info['url_prefix']}/roles/update",
    #         data=dict(role_id=new_role.id, role_name="update-role"),
    #         follow_redirects=True,
    #     )
    #     role = Role.query.scalar()

    #     assert response.status_code == 200
    #     assert request.path == f"{module_info['url_prefix']}/roles"
    #     assert b"Role successfully updated" in response.data
    #     assert role is not None
    #     assert role.name == "update-role"



"""
Specify main configs here

Other configs are specified in module's global.py in like
configs = {
   "development": {
       "CONFIG_VAR": "DEVVALUE"
   },
   "production": {
       "CONFIG_VAR": "PRODVALUE"
   },
   "testing": {
       "CONFIG_VAR": "TESTVALUE"
   }
}
"""
import os

base_path = os.path.dirname(os.path.abspath(__file__))


class BaseConfig:
    """Parent configuration class."""

    DEBUG = False
    SQLALCHEMY_TRACK_MODIFICATIONS = False
    BASE_DIR = base_path
    STATIC = os.path.join(base_path, "static")
    UPLOADED_PATH_IMAGE = os.path.join(STATIC, "uploads", "images")
    UPLOADED_PATH_THUMB = os.path.join(STATIC, "uploads", "thumbs")


class ProductionConfig(BaseConfig):
    """Configurations for production"""

    # built in flask configs
    ENV = "production"
    SECRET_KEY = os.environ.get("SECRET_KEY")

    # control email confirmation for user registration
    EMAIL_CONFIRMATION_DISABLED = False

    # flask-mailman configs
    MAIL_SERVER = "smtp.googlemail.com"
    MAIL_PORT = 465
    MAIL_USE_TLS = False
    MAIL_USE_SSL = True
    MAIL_USERNAME = os.environ.get("MAIL_USERNAME")
    MAIL_PASSWORD = os.environ.get("MAIL_PASSWORD")
    MAIL_DEFAULT_SENDER = os.environ.get("MAIL_DEFAULT_SENDER")

    # database configs
    SQLALCHEMY_DATABASE_URI = (
        os.environ.get("SQLALCHEMY_DATABASE_URI") or "sqlite:///shopyo.db"
    )


class DevelopmentConfig(BaseConfig):
    """Configurations for development"""

    # built in flask configs
    ENV = "development"
    DEBUG = True
    LOGIN_DISABLED = False
    SECRET_KEY = "secret"

    # control email confirmation for user registration
    EMAIL_CONFIRMATION_DISABLED = False

    # flask-mailman configs
    MAIL_SERVER = "localhost"
    MAIL_PORT = 1025
    MAIL_USE_TLS = False
    MAIL_USE_SSL = False
    MAIL_USERNAME = ""
    MAIL_PASSWORD = ""
    MAIL_DEFAULT_SENDER = "ma@mail.com"

    # database configs
    SQLALCHEMY_DATABASE_URI = "sqlite:///shopyo.db"

    # unknown configs
    PASSWORD_SALT = "some pasword salt"


class TestingConfig(BaseConfig):
    """Configurations for testsing"""

    # built in flask configs
    ENV = "testing"
    TESTING = True
    DEBUG = True
    SERVER_NAME = "localhost.com"
    SECRET_KEY = "secret"
    PREFERRED_URL_SCHEME = "http"

    # flask WTF configs
    WTF_CSRF_ENABLED = False

    # control email confirmation for user registration
    EMAIL_CONFIRMATION_DISABLED = False

    # flask-mailman configs
    MAIL_BACKEND = "console"
    MAIL_USERNAME = "shopyofrom@test.com"
    MAIL_PASSWORD = "pass"
    MAIL_DEFAULT_SENDER = "shopyofrom@test.com"

    # flask sqlalchemy configs
    SQLALCHEMY_DATABASE_URI = "sqlite:///testing.db"

    # flask bycrpt configs
    BCRYPT_LOG_ROUNDS = 4

    # unknown configs
    PASSWORD_SALT = "some pasword salt"
    LIVESERVER_PORT = 8943
    LIVESERVER_TIMEOUT = 10


app_config = {
    "development": DevelopmentConfig,
    "production": ProductionConfig,
    "testing": TestingConfig,
}



from flask import session

langs = {"en": "english", "fr": "french"}


def lang_keys():
    return (k for k in langs)


def get_current_lang():
    return session.get("yo_current_language", "en")


def get_default_lang():
    return session.get("yo_default_language", "en")



"""A setuptools based setup module.
See:
https://packaging.python.org/guides/distributing-packages-using-setuptools/
https://github.com/pypa/sampleproject


python setup.py publish to publish

"""
import os
import sys

from setuptools import setup

here = os.path.abspath(os.path.dirname(__file__))

if sys.argv[-1] == "publish":  # requests
    os.system("python setup.py sdist")  # bdist_wheel
    os.system("twine upload dist/* --skip-existing")
    sys.exit()

setup(
    install_requires=open(
        os.path.join(here, "requirements/requirements.txt"), encoding="utf-8"
    )
    .read()
    .split("\n"),
    url="https://github.com/shopyo/shopyo",
)



from flask_wtf import FlaskForm
from modules.box__default.i18n.helpers import langs
from wtforms import SelectField
from wtforms import StringField
from wtforms import TextAreaField
from wtforms.validators import DataRequired

from shopyo.api.validators import verify_slug

# from wtforms.validators import Length
# from wtforms.fields import EmailField


class PageForm(FlaskForm):
    content = TextAreaField(
        "Content",
        [],
        render_kw={
            "class": "form-control",
            "rows": "20",
            "autocomplete": "off",
        },
    )
    slug = StringField(
        "Slug",
        [DataRequired(), verify_slug],
        render_kw={"class": "form-control", "autocomplete": "off"},
    )
    title = StringField(
        "Title",
        [DataRequired()],
        render_kw={"class": "form-control", "autocomplete": "off"},
    )
    lang = SelectField("Language", choices=[(k, v) for k, v in langs.items()])



"""
File conftest.py contains pytest fixtures that are used in numerous
test functions. Refer to https://docs.pytest.org/en/stable/fixture.html
for more details on pytest
"""
import datetime
import json
import os

import pytest
from app import create_app
from flask import url_for
from flask_login import current_user as _current_user
from init import db as _db
from modules.box__default.auth.models import User
from modules.box__default.settings.models import Settings
from sqlalchemy import event

# run in shopyo/shopyo
# python -m pytest . or python -m pytest -v

if os.path.exists("testing.db"):
    os.remove("testing.db")


@pytest.fixture(scope="session")
def unconfirmed_user():
    """
    A pytest fixture that returns a non admin user
    """
    user = User()
    user.email = "unconfirmed@domain.com"
    user.password = "pass"
    user.is_email_confirmed = False
    return user


@pytest.fixture(scope="session")
def non_admin_user():
    """
    A pytest fixture that returns a non admin user
    """
    user = User()
    user.email = "admin1@domain.com"
    user.password = "pass"
    user.is_email_confirmed = True
    user.email_confirm_date = datetime.datetime.now()
    return user


@pytest.fixture(scope="session")
def admin_user():
    """
    A pytest fixture that returns an admin user
    """
    user = User()
    user.email = "admin2@domain.com"
    user.password = "pass"
    user.is_admin = True
    user.is_email_confirmed = True
    user.email_confirm_date = datetime.datetime.now()
    return user


@pytest.fixture(scope="session")
def flask_app():
    flask_app = create_app("testing")
    return flask_app


@pytest.fixture(scope="session")
def app(request):
    """
    Returns session-wide application.
    """
    return create_app("testing")


@pytest.fixture(scope="session")
def current_user():
    return _current_user


@pytest.fixture(scope="session")
def test_client(flask_app):
    """
    setups up and returns the flask testing app
    """
    # Create a test client using the Flask application configured for testing
    with flask_app.test_client() as testing_client:
        # Establish an application context
        with flask_app.app_context():
            yield testing_client  # this is where the testing happens!


@pytest.fixture(scope="session")
def db(test_client, non_admin_user, admin_user, unconfirmed_user):
    """
    creates and returns the initial testing database
    """
    # Create the database and the database table
    _db.app = test_client
    _db.create_all()

    # Insert admin, non admin, and unconfirmed
    _db.session.add(non_admin_user)
    _db.session.add(admin_user)
    _db.session.add(unconfirmed_user)

    # add the default settings
    with open("config.json") as config:
        config = json.load(config)
    for name, value in config["settings"].items():
        s = Settings(setting=name, value=value)
        _db.session.add(s)

    # Commit the changes for the users
    _db.session.commit()

    yield _db  # this is where the testing happens!

    _db.drop_all()


@pytest.fixture(scope="function", autouse=True)
def db_session(db):
    """
    Creates a new database session for a test. Note you must use this fixture
    if your test connects to db. Autouse is set to true which implies
    that the fixture will be setup before each test

    Here we not only support commit calls but also rollback calls in tests.
    """
    connection = db.engine.connect()
    transaction = connection.begin()
    options = dict(bind=connection, binds={})
    try:
        session = db._make_scoped_session(options=options)
    except:
        session = db.create_scoped_session(options=options)
    db.session = session

    yield session

    transaction.rollback()
    connection.close()
    session.remove()


# @pytest.fixture(scope="function", autouse=True)
# def db_session(app, db, request):
#     """
#     Returns function-scoped session.
#     """
#     with app.app_context():
#         conn = _db.engine.connect()
#         txn = conn.begin()

#         options = dict(bind=conn, binds={})
#         sess = _db.create_scoped_session(options=options)

#         # establish  a SAVEPOINT just before beginning the test
#         # (http://docs.sqlalchemy.org/en/latest/orm/session_transaction.html#using-savepoint)
#         sess.begin_nested()

#         @event.listens_for(sess(), "after_transaction_end")
#         def restart_savepoint(sess2, trans):
#             # Detecting whether this is indeed the nested transaction of the test
#             if trans.nested and not trans._parent.nested:
#                 # The test should have normally called session.commit(),
#                 # but to be safe we explicitly expire the session
#                 sess2.expire_all()
#                 sess.begin_nested()

#         _db.session = sess
#         yield sess

#         # Cleanup
#         sess.remove()
#         # This instruction rollsback any commit that were executed in the tests.
#         txn.rollback()
#         conn.close()


@pytest.fixture
def login_unconfirmed_user(auth, unconfirmed_user):
    """Login with unconfirmed and logout during teadown"""
    auth.login(unconfirmed_user)
    yield
    auth.logout()


@pytest.fixture
def login_admin_user(auth, admin_user):
    """Login with admin and logout during teadown"""
    auth.login(admin_user)
    yield
    auth.logout()


@pytest.fixture
def login_non_admin_user(auth, non_admin_user):
    """Login with non-admin and logout during teadown"""
    auth.login(non_admin_user)
    yield
    auth.logout()


@pytest.fixture
def auth(test_client):
    return AuthActions(test_client)


class AuthActions:
    def __init__(self, client):
        self._client = client

    def login(self, user, password="pass"):
        return self._client.post(
            url_for("auth.login"),
            data=dict(email=user.email, password=password),
            follow_redirects=True,
        )

    def logout(self):
        return self._client.get(url_for("auth.logout"), follow_redirects=True)


# Want TO USE THE BELOW 2 FIXTURES TO DYNAMICALLY
# GET THE ROUTES FOR A GIVEN MODULE BUT UNABLE TO
# PARAMETERIZE THE LIST OF ROUTES RETURNED FROM THE FIXTURE
# CURRENTLY THIS NOT POSSIBLE WITH FIXTURES IN PYTEST @rehmanis

# @pytest.fixture(scope="module")
# def get_module_routes(request, get_routes):
#     module_prefix = getattr(request.module, "module_prefix", "/")
#     return get_routes[module_prefix]


# @pytest.fixture(scope="session")
# def get_routes(flask_app):

#     routes_dict = {}
#     relative_path = "/"
#     prefix = "/"

#     for route in flask_app.url_map.iter_rules():
#         split_route = list(filter(None, str(route).split("/", 2)))

#         if len(split_route) == 0:
#             prefix = "/"
#             relative_path = ""
#         elif len(split_route) == 1:
#             prefix = "/" + split_route[0]
#             relative_path = "/"
#         else:
#             prefix = "/" + split_route[0]
#             relative_path = split_route[1]

#         if prefix in routes_dict:
#             routes_dict[prefix].append(relative_path)
#         else:
#             routes_dict[prefix] = [relative_path]

#     return routes_dict



"""
Used on flash
flash(notify_success('mail sent!'))
"""


def notify(message, alert_type="primary"):
    """
    Used with flash
        flash(notify('blabla'))

    Parameters
    ----------
    message: str
        message to be displayed

    alert_type: str
        bootstrap class

    Returns
    -------
    None
    """
    alert = """
    <div class="shopyo-alert alert alert-{alert_type} alert-dismissible
        fade show" role="alert" style="opacity: 0.98;">
      {message}

      <button type="button" class="close" data-dismiss="alert" aria-label="Close">
        <span aria-hidden="true">&times;</span>
      </button>
    </div>
    """.format(
        message=message, alert_type=alert_type
    )

    scriptFade = """
    <script>
          setTimeout(function() {
            $('#flashed-messages').fadeOut('fast');
        }, 5000); // <-- time in milliseconds (5 secs)
    </script>
    """
    return alert + scriptFade


def notify_success(message):
    return notify(message, alert_type="success")


def notify_danger(message):
    return notify(message, alert_type="danger")


def notify_warning(message):
    return notify(message, alert_type="warning")


def notify_info(message):
    return notify(message, alert_type="info")



"""
.. module:: AdminModels
   :synopsis: Contains model of a user Record

"""
import datetime

from flask import current_app
from flask_login import AnonymousUserMixin
from flask_login import UserMixin
from init import db
from init import login_manager
from itsdangerous import URLSafeTimedSerializer
from sqlalchemy.ext.hybrid import hybrid_property
from werkzeug.security import check_password_hash
from werkzeug.security import generate_password_hash

from shopyo.api.models import PkModel

role_user_bridge = db.Table(
    "role_user_bridge",
    db.Column(
        "user_id",
        db.Integer(),
        db.ForeignKey("users.id", ondelete="CASCADE"),
        primary_key=True,
    ),
    db.Column(
        "role_id",
        db.Integer(),
        db.ForeignKey("roles.id", ondelete="CASCADE"),
        primary_key=True,
    ),
)


class AnonymousUser(AnonymousUserMixin):
    """Anonymous user class"""

    def __init__(self):
        self.username = "guest"
        self.email = "<anonymous-user-no-email>"

    @property
    def is_email_confirmed(self):
        is_disabled = False

        if "EMAIL_CONFIRMATION_DISABLED" in current_app.config:
            is_disabled = current_app.config["EMAIL_CONFIRMATION_DISABLED"]

            if is_disabled is not True:
                is_disabled = False

        return is_disabled

    @property
    def is_admin(self):
        return False

    def __repr__(self):
        return f"<AnonymousUser {self.username}>"


login_manager.anonymous_user = AnonymousUser


class User(UserMixin, PkModel):
    """The user of the app"""

    __tablename__ = "users"

    username = db.Column(db.String(100), unique=True)
    _password = db.Column(db.String(128), nullable=False)
    first_name = db.Column(db.String(128))
    last_name = db.Column(db.String(128))
    is_admin = db.Column(db.Boolean, default=False)
    email = db.Column(db.String(120), unique=True, nullable=False)
    date_registered = db.Column(
        db.DateTime, nullable=False, default=datetime.datetime.now()
    )
    is_email_confirmed = db.Column(db.Boolean(), nullable=False, default=False)
    email_confirm_date = db.Column(db.DateTime)

    # A user can have many roles and a role can have many users
    roles = db.relationship(
        "Role",
        secondary=role_user_bridge,
        backref="users",
    )

    def __repr__(self):
        return f"<User-id: {self.id}, User-email: {self.email}>"

    @hybrid_property
    def password(self):
        return self._password

    @password.setter
    def password(self, plaintext):
        # the default hashing method is pbkdf2:sha256
        self._password = generate_password_hash(plaintext)

    def check_password(self, password):
        return check_password_hash(self._password, password)

    def generate_confirmation_token(self):
        serializer = URLSafeTimedSerializer(current_app.config["SECRET_KEY"])
        return serializer.dumps(self.email, salt=current_app.config["PASSWORD_SALT"])

    def confirm_token(self, token, expiration=3600):
        serializer = URLSafeTimedSerializer(current_app.config["SECRET_KEY"])
        email = False
        try:
            email = serializer.loads(
                token,
                salt=current_app.config["PASSWORD_SALT"],
                max_age=expiration,
            )
        except Exception as e:
            print(f"\nShopyo-LOG, Error at confirm_token: {e}")
            return False

        if email != self.email:
            return False

        self.is_email_confirmed = True
        self.email_confirm_date = datetime.datetime.now()
        self.update()
        return True


@login_manager.user_loader
def load_user(user_id):
    return User.query.get(user_id)


login_manager.login_view = "auth.login"


class Role(PkModel):
    """A role for a user."""

    __tablename__ = "roles"
    name = db.Column(db.String(100), nullable=False)

    def __repr__(self):
        return f"<Role-id: {self.id}, Role-name: {self.name}>"



"""
Tests all db utilities such as CRUDMixin defined under shopapi/models.py
Most of the test cases taken from:
https://github.com/cookiecutter-flask/cookiecutter-flask
"""
import pytest
from flask_login import UserMixin
from init import db

from shopyo.api.models import PkModel


class ExampleUserModel(PkModel, UserMixin):
    """Example user model for testing purposes"""

    __tablename__ = "testusers"

    username = db.Column(db.String(100), unique=True, nullable=False)
    email = db.Column(db.String(120), unique=True, nullable=False)


class TestPKModel:
    """Tests all functions of PKModel"""

    def test_get_by_id(self):
        result = ExampleUserModel.get_by_id("myId")

        assert not result

    def test_get_by_id_valid(self, db_session):
        user_bar = ExampleUserModel(username="bar", email="bar@domain.com")
        user_foo = ExampleUserModel(username="foo", email="foo@domain.com")
        user_bar.save()
        user_foo.save()
        result_int = ExampleUserModel.get_by_id(user_bar.id)
        result_str = ExampleUserModel.get_by_id(str(user_bar.id))
        expected = (
            db_session.query(ExampleUserModel)
            .filter(ExampleUserModel.username == "bar")
            .scalar()
        )

        assert result_int
        assert expected
        assert result_int.username == expected.username
        assert result_str.username == expected.username


class TestCRUDMixin:
    """Test class for testing all CRUD functions"""

    # def test_create(self, db_session):
    #     user = ExampleUserModel.create(username="bar", email="bar@domain.com")
    #     result_raw = db_session.execute("""select * from testusers""").fetchone()
    #     result_orm = (
    #         db_session.query(ExampleUserModel)
    #         .filter(ExampleUserModel.id == user.id)
    #         .scalar()
    #     )

    #     assert result_orm
    #     assert result_raw
    #     assert result_raw.username == "bar"
    #     assert result_orm.username == "bar"

    # @pytest.mark.parametrize("commit,expected", [(True, "foo"), (False, "bar")])
    # def test_update_single(self, db_session, commit, expected):
    #     user = ExampleUserModel(username="bar", email="bar@domain.com")
    #     user.save()
    #     user.update(commit=commit, username="foo")
    #     result = db_session.execute("""select * from testusers""").fetchone()

    #     assert result
    #     assert result.username == expected

    # @pytest.mark.parametrize(
    #     "commit,expected",
    #     [
    #         (True, {"username": "foo", "email": "foo@domain.com"}),
    #         (False, {"username": "bar", "email": "bar@domain.com"}),
    #     ],
    # )
    # def test_update_multiple(self, db_session, commit, expected):
    #     user = ExampleUserModel(username="bar", email="bar@domain.com")
    #     user.save()
    #     user.update(commit=commit, username="foo", email="foo@domain.com")
    #     result = db_session.execute("""select * from testusers""").fetchone()

    #     assert result
    #     assert result.username == expected["username"]
    #     assert result.email == expected["email"]

    # @pytest.mark.parametrize("commit,expected", [(True, None), (False, "bar")])
    # def test_delete(self, commit, expected):
    #     user = ExampleUserModel(username="bar", email="bar@domain.com")
    #     user.save()
    #     user.delete(commit=commit)
    #     result = ExampleUserModel.get_by_id(user.id)
    #     if result:
    #         result = result.username

    #     assert result == expected



import importlib
import os

from flask import current_app
from flask import send_from_directory
from flask import url_for


def get_static(boxormodule, filename):
    """
    Generates url for static file, depending on debug mode being on or not


    Args:
        boxormodule (String): box or module name e.g. box__default/auth or someothermodule/
        filename (String): path of the file inside the box or module

    Returns
    -------
    URL for static file
    """
    if current_app.config["DEBUG"] is True:
        return url_for("devstatic", boxormodule=boxormodule, path=filename)
    else:
        return url_for("static", path=f"modules/{boxormodule}/{filename}")


def register_devstatic(app, modules_path):
    """
    Resgisters endpoint for serving files in debug mode


    Args:
        app (Flask app)

    Returns
    -------
    URL for static file in debug mode
    """

    @app.route("/devstatic/<path:boxormodule>/f/<path:path>")
    def devstatic(boxormodule, path):
        if app.config["DEBUG"]:
            if boxormodule.startswith("box__"):
                box = boxormodule.split("/")[0]
                module = boxormodule.split("/")[1]
                module_static = os.path.join(modules_path, box, module, "static")
                return send_from_directory(module_static, path=path)
            if boxormodule.startswith("shopyo_"):
                plugin = importlib.import_module(boxormodule)
                plugin_folder_path = plugin.view.mhelp.dirpath
                plugin_static_folder = os.path.join(plugin_folder_path, "static")
                return send_from_directory(plugin_static_folder, path=path)
            else:
                module = boxormodule
                module_static = os.path.join(modules_path, module, "static")
                return send_from_directory(module_static, path=path)



# import pytest
# from shopyo.api import security
# class TestSecurity:
#     @pytest.mark.parametrize(
#         "url,expected",
#         [
#             ("https://domain.com?next=https://domain.com/somewhere", True),
#             ("https://domain.com?next=ftp://domain.com/somewhere", False),
#         ],
#     )
#     def test_is_safe_redirect_url(self, url, expected):
#         result = security.is_safe_redirect_url(url)
#         assert result == expected
